@@ -30,16 +30,17 @@
 #ifndef _LOADPARM_H
 #define _LOADPARM_H
 
+#include <talloc.h>
 #include "../lib/util/parmlist.h"
 
 /* the following are used by loadparm for option lists */
 typedef enum {
 	P_BOOL,P_BOOLREV,P_CHAR,P_INTEGER,P_OCTAL,P_LIST,
-	P_STRING,P_USTRING,P_ENUM,P_BYTES,P_CMDLIST,P_SEP
+	P_STRING,P_USTRING,P_ENUM,P_BYTES,P_CMDLIST
 } parm_type;
 
 typedef enum {
-	P_LOCAL,P_GLOBAL,P_SEPARATOR,P_NONE
+	P_LOCAL,P_GLOBAL,P_NONE
 } parm_class;
 
 struct enum_list {
@@ -48,7 +49,23 @@ struct enum_list {
 };
 
 struct loadparm_service;
-struct loadparm_context;
+struct loadparm_context {
+	const char *szConfigFile;
+	struct loadparm_global *globals;
+	struct loadparm_service **services;
+	struct loadparm_service *sDefault;
+	struct smb_iconv_handle *iconv_handle;
+	int iNumServices;
+	struct loadparm_service *currentService;
+	bool bInGlobalSection;
+	struct file_lists *file_lists;
+	unsigned int *flags;
+	bool loaded;
+	bool refuse_free;
+	bool global; /* Is this the global context, which may set
+		      * global variables such as debug level etc? */
+	const struct loadparm_s3_helpers *s3_fns;
+};
 
 struct parm_struct {
 	const char *label;
@@ -56,7 +73,7 @@ struct parm_struct {
 	parm_class p_class;
 	offset_t offset;
 	bool (*special)(struct loadparm_context *lpcfg_ctx,
-			int snum, const char *, char **);
+			struct loadparm_service *, const char *, char **);
 	const struct enum_list *enum_list;
 	unsigned flags;
 	union {
@@ -68,17 +85,17 @@ struct parm_struct {
 	} def;
 };
 
-/* The following flags are used in SWAT */
-#define FLAG_BASIC 	0x0001 /* Display only in BASIC view */
-#define FLAG_SHARE 	0x0002 /* file sharing options */
-#define FLAG_PRINT 	0x0004 /* printing options */
-#define FLAG_GLOBAL 	0x0008 /* local options that should be globally settable in SWAT */
-#define FLAG_WIZARD 	0x0010 /* Parameters that the wizard will operate on */
-#define FLAG_ADVANCED 	0x0020 /* Parameters that will be visible in advanced view */
-#define FLAG_DEVELOPER 	0x0040 /* No longer used */
+extern struct parm_struct parm_table[];
+
+struct file_lists {
+	struct file_lists *next;
+	char *name;
+	char *subfname;
+	time_t modtime;
+};
+
 #define FLAG_DEPRECATED 0x1000 /* options that should no longer be used */
-#define FLAG_HIDE  	0x2000 /* options that should be hidden in SWAT */
-#define FLAG_META	0x8000 /* A meta directive - not a real parameter */
+#define FLAG_SYNONYM	0x2000 /* options that is a synonym of another option */
 #define FLAG_CMDLINE	0x10000 /* option has been overridden */
 #define FLAG_DEFAULT    0x20000 /* this option was a default */
 
@@ -176,6 +193,12 @@ enum printing_types {PRINT_BSD,PRINT_SYSV,PRINT_AIX,PRINT_HPUX,
 #define ADS_AUTH_SASL_FORCE       0x0080
 #define ADS_AUTH_USER_CREDS       0x0100
 
+enum ldap_server_require_strong_auth {
+	LDAP_SERVER_REQUIRE_STRONG_AUTH_NO,
+	LDAP_SERVER_REQUIRE_STRONG_AUTH_ALLOW_SASL_OVER_TLS,
+	LDAP_SERVER_REQUIRE_STRONG_AUTH_YES,
+};
+
 /* DNS update settings */
 enum dns_update_settings {DNS_UPDATE_OFF, DNS_UPDATE_ON, DNS_UPDATE_SIGNED};
 
@@ -203,6 +226,17 @@ enum case_handling {CASE_LOWER,CASE_UPPER};
 #define PRINT_MAX_JOBID 10000
 #endif
 
+/* the default guest account - allow override via CFLAGS */
+#ifndef GUEST_ACCOUNT
+#define GUEST_ACCOUNT "nobody"
+#endif
+
+/* SMB2 defaults */
+#define DEFAULT_SMB2_MAX_READ (8*1024*1024)
+#define DEFAULT_SMB2_MAX_WRITE (8*1024*1024)
+#define DEFAULT_SMB2_MAX_TRANSACT (8*1024*1024)
+#define DEFAULT_SMB2_MAX_CREDITS 8192
+
 #define LOADPARM_EXTRA_LOCALS						\
 	bool valid;						        \
 	int usershare;							\
@@ -219,9 +253,37 @@ enum case_handling {CASE_LOWER,CASE_UPPER};
 
 #include "lib/param/param_local.h"
 
+#define LOADPARM_EXTRA_GLOBALS \
+	struct parmlist_entry *param_opt;				\
+	char *realm_original;						\
+	int iminreceivefile;						\
+	char *szPrintcapname;						\
+	int CupsEncrypt;						\
+	int  iPreferredMaster;						\
+	char *szLdapMachineSuffix;					\
+	char *szLdapUserSuffix;						\
+	char *szLdapIdmapSuffix;					\
+	char *szLdapGroupSuffix;					\
+	char *szIdmapUID;						\
+	char *szIdmapGID;						\
+	char *szIdmapBackend;						\
+	int winbindMaxDomainConnections;				\
+	int ismb2_max_credits;
+
 const char* server_role_str(uint32_t role);
 int lp_find_server_role(int server_role, int security, int domain_logons, int domain_master);
 int lp_find_security(int server_role, int security);
 bool lp_is_security_and_server_role_valid(int server_role, int security);
 
+struct loadparm_global * get_globals(void);
+unsigned int * get_flags(void);
+char * lp_string(TALLOC_CTX *, const char *);
+int getservicebyname(const char *, struct loadparm_service *);
+bool lp_include(struct loadparm_context *, struct loadparm_service *,
+	       	const char *, char **);
+bool lp_do_section(const char *pszSectionName, void *userdata);
+bool store_lp_set_cmdline(const char *pszParmName, const char *pszParmValue);
+
+int num_parameters(void);
+
 #endif /* _LOADPARM_H */
