@@ -24,9 +24,10 @@
 #include "includes.h"
 #include "system/filesys.h"
 #include "system/locale.h"
-#if HAVE_SYSTEMD
+#if HAVE_LIBSYSTEMD_DAEMON
 #include <systemd/sd-daemon.h>
 #endif
+#include "lib/util/close_low_fd.h"
 
 /*******************************************************************
  Close the low 3 fd's and open dev/null in their place.
@@ -34,42 +35,28 @@
 
 _PUBLIC_ void close_low_fds(bool stdin_too, bool stdout_too, bool stderr_too)
 {
-#ifndef VALGRIND
-	int fd;
-	int i;
 
-	if (stdin_too)
-		close(0);
-	if (stdout_too)
-		close(1);
-
-	if (stderr_too)
-		close(2);
-
-	/* try and use up these file descriptors, so silly
-		library routines writing to stdout etc won't cause havoc */
-	for (i=0;i<3;i++) {
-		if (i == 0 && !stdin_too)
-			continue;
-		if (i == 1 && !stdout_too)
-			continue;
-		if (i == 2 && !stderr_too)
-			continue;
-
-		fd = open("/dev/null",O_RDWR,0);
-		if (fd < 0)
-			fd = open("/dev/null",O_WRONLY,0);
-		if (fd < 0) {
-			DEBUG(0,("Can't open /dev/null\n"));
-			return;
+	if (stdin_too) {
+		int ret = close_low_fd(0);
+		if (ret != 0) {
+			DEBUG(0, ("%s: close_low_fd(0) failed: %s\n",
+				  __func__, strerror(ret)));
 		}
-		if (fd != i) {
-			DEBUG(0,("Didn't get file descriptor %d\n",i));
-			close(fd);
-			return;
+	}
+	if (stdout_too) {
+		int ret = close_low_fd(1);
+		if (ret != 0) {
+			DEBUG(0, ("%s: close_low_fd(1) failed: %s\n",
+				  __func__, strerror(ret)));
+		}
+	}
+	if (stderr_too) {
+		int ret = close_low_fd(2);
+		if (ret != 0) {
+			DEBUG(0, ("%s: close_low_fd(2) failed: %s\n",
+				  __func__, strerror(ret)));
 		}
 	}
-#endif
 }
 
 /****************************************************************************
@@ -82,9 +69,9 @@ _PUBLIC_ void become_daemon(bool do_fork, bool no_process_group, bool log_stdout
 	if (do_fork) {
 		newpid = fork();
 		if (newpid) {
-#if HAVE_SYSTEMD
+#if HAVE_LIBSYSTEMD_DAEMON
 			sd_notifyf(0, "READY=0\nSTATUS=Starting process...\nMAINPID=%lu", (unsigned long) newpid);
-#endif /* HAVE_SYSTEMD */
+#endif /* HAVE_LIBSYSTEMD_DAEMON */
 			_exit(0);
 		}
 	}
@@ -111,7 +98,7 @@ _PUBLIC_ void become_daemon(bool do_fork, bool no_process_group, bool log_stdout
 
 _PUBLIC_ void exit_daemon(const char *msg, int error)
 {
-#ifdef HAVE_SYSTEMD
+#ifdef HAVE_LIBSYSTEMD_DAEMON
 	if (msg == NULL) {
 		msg = strerror(error);
 	}
@@ -125,15 +112,16 @@ _PUBLIC_ void exit_daemon(const char *msg, int error)
 	exit(1);
 }
 
-_PUBLIC_ void daemon_ready(const char *daemon)
+_PUBLIC_ void daemon_ready(const char *name)
 {
-	if (daemon == NULL) {
-		daemon = "Samba";
+	if (name == NULL) {
+		name = "Samba";
 	}
-#ifdef HAVE_SYSTEMD
-	sd_notifyf(0, "READY=1\nSTATUS=%s: ready to serve connections...", daemon);
+#ifdef HAVE_LIBSYSTEMD_DAEMON
+	sd_notifyf(0, "READY=1\nSTATUS=%s: ready to serve connections...", name);
 #endif
-	DEBUG(0, ("STATUS=daemon '%s' finished starting up and ready to serve connections", daemon));
+	DEBUG(0, ("STATUS=daemon '%s' finished starting up and ready to serve "
+		  "connections\n", name));
 }
 
 _PUBLIC_ void daemon_status(const char *name, const char *msg)
@@ -141,7 +129,7 @@ _PUBLIC_ void daemon_status(const char *name, const char *msg)
 	if (name == NULL) {
 		name = "Samba";
 	}
-#ifdef HAVE_SYSTEMD
+#ifdef HAVE_LIBSYSTEMD_DAEMON
 	sd_notifyf(0, "\nSTATUS=%s: %s", name, msg);
 #endif
 	DEBUG(0, ("STATUS=daemon '%s' : %s", name, msg));
