@@ -24,9 +24,7 @@
 #include "../lib/tsocket/tsocket_internal.h"
 #include "../librpc/gen_ndr/ndr_named_pipe_auth.h"
 #include "../libcli/named_pipe_auth/npa_tstream.h"
-#if _SAMBA_BUILD_ == 4
-#include "libcli/raw/smb.h"
-#endif
+#include "../libcli/smb/smb_constants.h"
 
 static const struct tstream_context_ops tstream_npa_ops;
 
@@ -1323,7 +1321,7 @@ static void tstream_npa_accept_existing_reply(struct tevent_req *subreq)
 						i4.client_port,
 						&state->client);
 	if (ret != 0) {
-		DEBUG(2, ("Invalid server address[%s:%u] - %s\n",
+		DEBUG(2, ("Invalid client address[%s:%u] - %s\n",
 			  i4.client_addr, i4.client_port,
 			  strerror(errno)));
 		pipe_reply.status = NT_STATUS_INVALID_ADDRESS;
@@ -1444,3 +1442,86 @@ int _tstream_npa_accept_existing_recv(struct tevent_req *req,
 	tevent_req_received(req);
 	return 0;
 }
+
+
+/* SOCKETPAIR for internal rpc communication */
+
+/* file_type is FILE_TYPE_BYTE_MODE_PIPE or FILE_TYPE_MESSAGE_MODE_PIPE */
+int _tstream_npa_socketpair(uint16_t file_type,
+			    TALLOC_CTX *mem_ctx1,
+			    struct tstream_context **pstream1,
+			    TALLOC_CTX *mem_ctx2,
+			    struct tstream_context **pstream2,
+			    const char *location)
+{
+	struct tstream_context *stream1 = NULL;
+	struct tstream_context *stream2 = NULL;
+	int fds[2];
+	int fd1;
+	int fd2;
+	int rc;
+	bool ok;
+
+	rc = socketpair(AF_UNIX, SOCK_STREAM, 0, fds);
+	if (rc == -1) {
+		return -1;
+	}
+	fd1 = fds[0];
+	fd2 = fds[1];
+
+	ok = smb_set_close_on_exec(fd1);
+	if (!ok) {
+		goto close_fail;
+	}
+
+	ok = smb_set_close_on_exec(fd2);
+	if (!ok) {
+		goto close_fail;
+	}
+
+	rc = set_blocking(fd1, false);
+	if (rc == -1) {
+		goto close_fail;
+	}
+
+	rc = set_blocking(fd2, false);
+	if (rc == -1) {
+		goto close_fail;
+	}
+
+	rc = _tstream_npa_existing_socket(mem_ctx1,
+					  fd1,
+					  file_type,
+					  &stream1,
+					  location);
+	if (rc == -1) {
+		goto close_fail;
+	}
+
+	rc = _tstream_npa_existing_socket(mem_ctx2,
+					  fd2,
+					  file_type,
+					  &stream2,
+					  location);
+	if (rc == -1) {
+		int sys_errno = errno;
+		talloc_free(stream1);
+		close(fd2);
+		errno = sys_errno;
+		return -1;
+	}
+
+	*pstream1 = stream1;
+	*pstream2 = stream2;
+
+	return 0;
+
+close_fail:
+	{
+		int sys_errno = errno;
+		close(fd1);
+		close(fd2);
+		errno = sys_errno;
+		return -1;
+	}
+}
