@@ -26,10 +26,6 @@
 
 void initnetbios(void);
 
-#ifndef Py_RETURN_NONE
-#define Py_RETURN_NONE return Py_INCREF(Py_None), Py_None
-#endif
-
 extern PyTypeObject nbt_node_Type;
 
 typedef struct {
