@@ -1,28 +1,27 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
    Samba utility functions
    Copyright (C) Andrew Tridgell 1992-1999
    Copyright (C) Luke Kenneth Casson Leighton 1996 - 1999
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
-   This program is distributed in the hope that it will be useful, 
+
+   This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "includes.h"
-#include "librpc/gen_ndr/security.h"
-#include "libcli/security/secace.h"
-#include "libcli/security/dom_sid.h"
+#include "libcli/security/security.h"
 #include "librpc/ndr/libndr.h"
+#include "libcli/security/display_sec.h"
 
 /****************************************************************************
 convert a security permissions into a string
@@ -159,15 +158,20 @@ void display_sec_ace_flags(uint8_t flags)
  ****************************************************************************/
 static void disp_sec_ace_object(struct security_ace_object *object)
 {
+	char *str;
 	if (object->flags & SEC_ACE_OBJECT_TYPE_PRESENT) {
+		str = GUID_string(NULL, &object->type.type);
+		if (str == NULL) return;
 		printf("Object type: SEC_ACE_OBJECT_TYPE_PRESENT\n");
-		printf("Object GUID: %s\n", GUID_string(talloc_tos(),
-			&object->type.type));
+		printf("Object GUID: %s\n", str);
+		talloc_free(str);
 	}
 	if (object->flags & SEC_ACE_INHERITED_OBJECT_TYPE_PRESENT) {
+		str = GUID_string(NULL, &object->inherited_type.inherited_type);
+		if (str == NULL) return;
 		printf("Object type: SEC_ACE_INHERITED_OBJECT_TYPE_PRESENT\n");
-		printf("Object GUID: %s\n", GUID_string(talloc_tos(), 
-			&object->inherited_type.inherited_type));
+		printf("Object GUID: %s\n", str);
+		talloc_free(str);
 	}
 }
 
@@ -230,10 +234,10 @@ void display_sec_ace(struct security_ace *ace)
  ****************************************************************************/
 void display_sec_acl(struct security_acl *sec_acl)
 {
-	int i;
+	uint32_t i;
 
-	printf("\tACL\tNum ACEs:\t%d\trevision:\t%x\n",
-			 sec_acl->num_aces, sec_acl->revision); 
+	printf("\tACL\tNum ACEs:\t%u\trevision:\t%x\n",
+	       sec_acl->num_aces, sec_acl->revision);
 	printf("\t---\n");
 
 	if (sec_acl->size != 0 && sec_acl->num_aces != 0) {
@@ -279,7 +283,7 @@ void display_acl_type(uint16_t type)
 		printf("SEC_DESC_RM_CONTROL_VALID ");
 	if (type & SEC_DESC_SELF_RELATIVE)	/* 0x8000 */
 		printf("SEC_DESC_SELF_RELATIVE ");
-	
+
 	printf("\n");
 }
 
