@@ -27,7 +27,7 @@
  */
 
 #ifndef MAX_OPEN_FUDGEFACTOR
-#define MAX_OPEN_FUDGEFACTOR 20
+#define MAX_OPEN_FUDGEFACTOR 40
 #endif
 
 /*
@@ -80,11 +80,6 @@
    NT file manager window? */
 #define FSTYPE_STRING "NTFS"
 
-/* the default guest account - normally set in the Makefile or smb.conf */
-#ifndef GUEST_ACCOUNT
-#define GUEST_ACCOUNT "nobody"
-#endif
-
 /* user to test password server with as invalid in security=server mode. */
 #ifndef INVALID_USER_PREFIX
 #define INVALID_USER_PREFIX "sambatest"
@@ -209,14 +204,8 @@
 /* Maximum size of RPC data we will accept for one call. */
 #define MAX_RPC_DATA_SIZE (15*1024*1024)
 
-#define CLIENT_NDR_PADDING_SIZE 8
-#define SERVER_NDR_PADDING_SIZE 8
-
-#define DEFAULT_SMB2_MAX_READ (1024*1024)
-#define DEFAULT_SMB2_MAX_WRITE (1024*1024)
-#define DEFAULT_SMB2_MAX_TRANSACT (1024*1024)
-#define DEFAULT_SMB2_MAX_CREDITS 8192
-
-#define WINDOWS_CLIENT_PURE_SMB2_NEGPROT_INITIAL_CREDIT_ASK	31
-
+/* A guestimate of how many domains winbindd will be contacting */
+#ifndef WINBIND_MAX_DOMAINS_HINT
+#define WINBIND_MAX_DOMAINS_HINT 10
+#endif
 #endif
