@@ -210,7 +210,8 @@ static bool init_mappings(connection_struct *conn,
 	if (share_level->mappings) {
 		(*selected_out) = share_level;
 		return True;
-	} else if (global->mappings) {
+	}
+	if (global->mappings) {
 		share_level->mappings = global->mappings;
 		(*selected_out) = share_level;
 		return True;
@@ -270,7 +271,7 @@ static NTSTATUS catia_string_replace_allocate(connection_struct *conn,
 static DIR *catia_opendir(vfs_handle_struct *handle,
 				     const char *fname,
 				     const char *mask,
-				     uint32 attr)
+				     uint32_t attr)
 {
 	char *name_mapped = NULL;
 	NTSTATUS status;
@@ -549,6 +550,25 @@ static int catia_lchown(vfs_handle_struct *handle,
 	return ret;
 }
 
+static int catia_chmod(vfs_handle_struct *handle, const char *path, mode_t mode)
+{
+	char *name = NULL;
+	NTSTATUS status;
+	int ret;
+
+	status = catia_string_replace_allocate(handle->conn, path,
+					&name, vfs_translate_to_unix);
+	if (!NT_STATUS_IS_OK(status)) {
+		errno = map_errno_from_nt_status(status);
+		return -1;
+	}
+
+	ret = SMB_VFS_NEXT_CHMOD(handle, name, mode);
+	TALLOC_FREE(name);
+
+	return ret;
+}
+
 static int catia_rmdir(vfs_handle_struct *handle,
 		       const char *path)
 {
@@ -686,11 +706,17 @@ catia_streaminfo(struct vfs_handle_struct *handle,
 		 struct files_struct *fsp,
 		 const char *path,
 		 TALLOC_CTX *mem_ctx,
-		 unsigned int *num_streams,
-		 struct stream_struct **streams)
+		 unsigned int *_num_streams,
+		 struct stream_struct **_streams)
 {
 	char *mapped_name = NULL;
 	NTSTATUS status;
+	int i;
+	unsigned int num_streams = 0;
+	struct stream_struct *streams = NULL;
+
+	*_num_streams = 0;
+	*_streams = NULL;
 
 	status = catia_string_replace_allocate(handle->conn, path,
 				        &mapped_name, vfs_translate_to_unix);
@@ -700,16 +726,60 @@ catia_streaminfo(struct vfs_handle_struct *handle,
 	}
 
 	status = SMB_VFS_NEXT_STREAMINFO(handle, fsp, mapped_name,
-					 mem_ctx, num_streams,streams);
+					 mem_ctx, &num_streams, &streams);
 	TALLOC_FREE(mapped_name);
+	if (!NT_STATUS_IS_OK(status)) {
+		return status;
+	}
 
-	return status;
+	/*
+	 * Translate stream names just like the base names
+	 */
+	for (i = 0; i < num_streams; i++) {
+		/*
+		 * Strip ":" prefix and ":$DATA" suffix to get a
+		 * "pure" stream name and only translate that.
+		 */
+		void *old_ptr = streams[i].name;
+		char *stream_name = streams[i].name + 1;
+		char *stream_type = strrchr_m(stream_name, ':');
+
+		if (stream_type != NULL) {
+			*stream_type = '\0';
+			stream_type += 1;
+		}
+
+		status = catia_string_replace_allocate(handle->conn, stream_name,
+						       &mapped_name, vfs_translate_to_windows);
+		if (!NT_STATUS_IS_OK(status)) {
+			TALLOC_FREE(streams);
+			return status;
+		}
+
+		if (stream_type != NULL) {
+			streams[i].name = talloc_asprintf(streams, ":%s:%s",
+							  mapped_name, stream_type);
+		} else {
+			streams[i].name = talloc_asprintf(streams, ":%s",
+							  mapped_name);
+		}
+		TALLOC_FREE(mapped_name);
+		TALLOC_FREE(old_ptr);
+		if (streams[i].name == NULL) {
+			TALLOC_FREE(streams);
+			return NT_STATUS_NO_MEMORY;
+		}
+	}
+
+	*_num_streams = num_streams;
+	*_streams = streams;
+	return NT_STATUS_OK;
 }
 
 static NTSTATUS
 catia_get_nt_acl(struct vfs_handle_struct *handle,
 		 const char *path,
-		 uint32 security_info,
+		 uint32_t security_info,
 		 TALLOC_CTX *mem_ctx,
 		 struct security_descriptor **ppdesc)
 {
@@ -917,6 +987,7 @@ static struct vfs_fn_pointers vfs_catia_fns = {
 	.unlink_fn = catia_unlink,
 	.chown_fn = catia_chown,
 	.lchown_fn = catia_lchown,
+	.chmod_fn = catia_chmod,
 	.chdir_fn = catia_chdir,
 	.ntimes_fn = catia_ntimes,
 	.realpath_fn = catia_realpath,
