@@ -81,13 +81,13 @@ static void wins_delete_all_1b_in_memory_records(void)
 static struct name_record *wins_record_to_name_record(TDB_DATA key, TDB_DATA data)
 {
 	struct name_record *namerec = NULL;
-	uint16 nb_flags;
+	uint16_t nb_flags;
 	unsigned char nr_src;
-	uint32 death_time, refresh_time;
-	uint32 id_low, id_high;
-	uint32 saddr;
-	uint32 wins_flags;
-	uint32 num_ips;
+	uint32_t death_time, refresh_time;
+	uint32_t id_low, id_high;
+	uint32_t saddr;
+	uint32_t wins_flags;
+	uint32_t num_ips;
 	size_t len;
 	int i;
 
@@ -146,9 +146,7 @@ static struct name_record *wins_record_to_name_record(TDB_DATA key, TDB_DATA dat
 	namerec->data.death_time = (time_t)death_time;
 	namerec->data.refresh_time = (time_t)refresh_time;
 	namerec->data.id = id_low;
-#if defined(HAVE_LONGLONG)
 	namerec->data.id |= ((uint64_t)id_high << 32);
-#endif
 	namerec->data.wins_ip.s_addr = saddr;
 	namerec->data.wins_flags = wins_flags,
 	namerec->data.num_ips = num_ips;
@@ -169,19 +167,15 @@ static TDB_DATA name_record_to_wins_record(const struct name_record *namerec)
 	TDB_DATA data;
 	size_t len = 0;
 	int i;
-	uint32 id_low = (namerec->data.id & 0xFFFFFFFF);
-#if defined(HAVE_LONGLONG)
-	uint32 id_high = (namerec->data.id >> 32) & 0xFFFFFFFF;
-#else
-	uint32 id_high = 0;
-#endif
+	uint32_t id_low = (namerec->data.id & 0xFFFFFFFF);
+	uint32_t id_high = (namerec->data.id >> 32) & 0xFFFFFFFF;
 
 	ZERO_STRUCT(data);
 
 	len = (2 + 1 + (7*4)); /* "wbddddddd" */
 	len += (namerec->data.num_ips * 4);
 
-	data.dptr = (uint8 *)SMB_MALLOC(len);
+	data.dptr = (uint8_t *)SMB_MALLOC(len);
 	if (!data.dptr) {
 		return data;
 	}
@@ -190,13 +184,13 @@ static TDB_DATA name_record_to_wins_record(const struct name_record *namerec)
 	len = tdb_pack(data.dptr, data.dsize, "wbddddddd",
                         namerec->data.nb_flags,
                         (unsigned char)namerec->data.source,
-                        (uint32)namerec->data.death_time,
-                        (uint32)namerec->data.refresh_time,
+                        (uint32_t)namerec->data.death_time,
+                        (uint32_t)namerec->data.refresh_time,
                         id_low,
                         id_high,
-                        (uint32)namerec->data.wins_ip.s_addr,
-                        (uint32)namerec->data.wins_flags,
-                        (uint32)namerec->data.num_ips );
+                        (uint32_t)namerec->data.wins_ip.s_addr,
+                        (uint32_t)namerec->data.wins_flags,
+                        (uint32_t)namerec->data.num_ips );
 
 	for (i = 0; i < namerec->data.num_ips; i++) {
 		SIVAL(data.dptr, len + (i*4), namerec->data.ip[i].s_addr);
@@ -219,7 +213,7 @@ static TDB_DATA name_to_key(const struct nmb_name *nmbname)
 	pull_ascii_nstring(keydata, sizeof(unstring), nmbname->name);
 	(void)strupper_m(keydata);
 	keydata[sizeof(unstring)] = nmbname->name_type;
-	key.dptr = (uint8 *)keydata;
+	key.dptr = (uint8_t *)keydata;
 	key.dsize = sizeof(keydata);
 
 	return key;
@@ -241,7 +235,7 @@ struct name_record *find_name_on_wins_subnet(const struct nmb_name *nmbname, boo
 	}
 
 	key = name_to_key(nmbname);
-	data = tdb_fetch_compat(wins_tdb, key);
+	data = tdb_fetch(wins_tdb, key);
 
 	if (data.dsize == 0) {
 		return NULL;
@@ -600,14 +594,22 @@ bool initialise_wins(void)
 	time_t time_now = time(NULL);
 	XFILE *fp;
 	char line[1024];
+	char *db_path;
+	char *list_path;
 
 	if(!lp_we_are_a_wins_server()) {
 		return True;
 	}
 
+	db_path = state_path("wins.tdb");
+	if (db_path == NULL) {
+		return false;
+	}
+
 	/* Open the wins.tdb. */
-	wins_tdb = tdb_open_log(state_path("wins.tdb"), 0, TDB_DEFAULT|TDB_CLEAR_IF_FIRST|TDB_INCOMPATIBLE_HASH,
+	wins_tdb = tdb_open_log(db_path, 0, TDB_DEFAULT|TDB_CLEAR_IF_FIRST|TDB_INCOMPATIBLE_HASH,
 			O_CREAT|O_RDWR, 0600);
+	TALLOC_FREE(db_path);
 	if (!wins_tdb) {
 		DEBUG(0,("initialise_wins: failed to open wins.tdb. Error was %s\n",
 			strerror(errno) ));
@@ -618,7 +620,15 @@ bool initialise_wins(void)
 
 	add_samba_names_to_subnet(wins_server_subnet);
 
-	if((fp = x_fopen(state_path(WINS_LIST),O_RDONLY,0)) == NULL) {
+	list_path = state_path(WINS_LIST);
+	if (list_path == NULL) {
+		tdb_close(wins_tdb);
+		return false;
+	}
+
+	fp = x_fopen(list_path, O_RDONLY, 0);
+	TALLOC_FREE(list_path);
+	if (fp == NULL) {
 		DEBUG(2,("initialise_wins: Can't open wins database file %s. Error was %s\n",
 			WINS_LIST, strerror(errno) ));
 		return True;
@@ -852,7 +862,7 @@ void wins_process_name_refresh_request( struct subnet_record *subrec,
 	struct nmb_packet *nmb = &p->packet.nmb;
 	struct nmb_name *question = &nmb->question.question_name;
 	bool bcast = nmb->header.nm_flags.bcast;
-	uint16 nb_flags = get_nb_flags(nmb->additional->rdata);
+	uint16_t nb_flags = get_nb_flags(nmb->additional->rdata);
 	bool group = (nb_flags & NB_GROUP) ? True : False;
 	struct name_record *namerec = NULL;
 	int ttl = get_ttl_from_packet(nmb);
@@ -1159,7 +1169,7 @@ void wins_process_name_registration_request(struct subnet_record *subrec,
 	struct nmb_packet *nmb = &p->packet.nmb;
 	struct nmb_name *question = &nmb->question.question_name;
 	bool bcast = nmb->header.nm_flags.bcast;
-	uint16 nb_flags = get_nb_flags(nmb->additional->rdata);
+	uint16_t nb_flags = get_nb_flags(nmb->additional->rdata);
 	int ttl = get_ttl_from_packet(nmb);
 	struct name_record *namerec = NULL;
 	struct in_addr from_ip;
@@ -1609,7 +1619,7 @@ void wins_process_multihomed_name_registration_request( struct subnet_record *su
 	struct nmb_packet *nmb = &p->packet.nmb;
 	struct nmb_name *question = &nmb->question.question_name;
 	bool bcast = nmb->header.nm_flags.bcast;
-	uint16 nb_flags = get_nb_flags(nmb->additional->rdata);
+	uint16_t nb_flags = get_nb_flags(nmb->additional->rdata);
 	int ttl = get_ttl_from_packet(nmb);
 	struct name_record *namerec = NULL;
 	struct in_addr from_ip;
@@ -2137,7 +2147,7 @@ void wins_process_name_release_request(struct subnet_record *subrec,
 	struct nmb_packet *nmb = &p->packet.nmb;
 	struct nmb_name *question = &nmb->question.question_name;
 	bool bcast = nmb->header.nm_flags.bcast;
-	uint16 nb_flags = get_nb_flags(nmb->additional->rdata);
+	uint16_t nb_flags = get_nb_flags(nmb->additional->rdata);
 	struct name_record *namerec = NULL;
 	struct in_addr from_ip;
 	bool releasing_group_name = (nb_flags & NB_GROUP) ? True : False;
