@@ -4,22 +4,30 @@
    Copyright (C) Andrew Tridgell         1992-1998
    Copyright (C) Gerald (Jerry) Carter   2003
    Copyright (C) Volker Lendecke	 2005
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "includes.h"
+#include "passdb.h"
+#include "../librpc/gen_ndr/ndr_security.h"
+#include "secrets.h"
+#include "../lib/util/memcache.h"
+#include "idmap_cache.h"
+#include "../libcli/security/security.h"
+#include "lib/winbind_util.h"
+#include "../librpc/gen_ndr/idmap.h"
 
 /*****************************************************************
  Dissect a user-provided name into domain, name, sid and type.
@@ -32,14 +40,14 @@
 bool lookup_name(TALLOC_CTX *mem_ctx,
 		 const char *full_name, int flags,
 		 const char **ret_domain, const char **ret_name,
-		 DOM_SID *ret_sid, enum lsa_SidType *ret_type)
+		 struct dom_sid *ret_sid, enum lsa_SidType *ret_type)
 {
 	char *p;
 	const char *tmp;
 	const char *domain = NULL;
 	const char *name = NULL;
-	uint32 rid;
-	DOM_SID sid;
+	uint32_t rid;
+	struct dom_sid sid;
 	enum lsa_SidType type;
 	TALLOC_CTX *tmp_ctx = talloc_new(mem_ctx);
 
@@ -65,7 +73,7 @@ bool lookup_name(TALLOC_CTX *mem_ctx,
 		return false;
 	}
 
-	DEBUG(10,("lookup_name: %s => %s (domain), %s (name)\n",
+	DEBUG(10,("lookup_name: %s => domain=[%s], name=[%s]\n",
 		full_name, domain, name));
 	DEBUG(10, ("lookup_name: flags = 0x0%x\n", flags));
 
@@ -75,8 +83,7 @@ bool lookup_name(TALLOC_CTX *mem_ctx,
 
 		/* It's our own domain, lookup the name in passdb */
 		if (lookup_global_sam_name(name, flags, &rid, &type)) {
-			sid_copy(&sid, get_global_sam_sid());
-			sid_append_rid(&sid, rid);
+			sid_compose(&sid, get_global_sam_sid(), rid);
 			goto ok;
 		}
 		TALLOC_FREE(tmp_ctx);
@@ -96,8 +103,7 @@ bool lookup_name(TALLOC_CTX *mem_ctx,
 
 		/* Explicit request for a name in BUILTIN */
 		if (lookup_builtin_name(name, &rid)) {
-			sid_copy(&sid, &global_sid_Builtin);
-			sid_append_rid(&sid, rid);
+			sid_compose(&sid, &global_sid_Builtin, rid);
 			type = SID_NAME_ALIAS;
 			goto ok;
 		}
@@ -114,7 +120,7 @@ bool lookup_name(TALLOC_CTX *mem_ctx,
 			goto ok;
 	}
 
-	if (((flags & LOOKUP_NAME_NO_NSS) == 0)
+	if (((flags & (LOOKUP_NAME_NO_NSS|LOOKUP_NAME_GROUP)) == 0)
 	    && strequal(domain, unix_users_domain_name())) {
 		if (lookup_unix_user_name(name, &sid)) {
 			type = SID_NAME_USER;
@@ -134,7 +140,31 @@ bool lookup_name(TALLOC_CTX *mem_ctx,
 		return false;
 	}
 
-	if ((domain[0] == '\0') && (!(flags & LOOKUP_NAME_ISOLATED))) {
+	/*
+	 * Finally check for a well known domain name ("NT Authority"),
+	 * this is taken care if in lookup_wellknown_name().
+	 */
+	if ((domain[0] != '\0') &&
+	    (flags & LOOKUP_NAME_WKN) &&
+	    lookup_wellknown_name(tmp_ctx, name, &sid, &domain))
+	{
+		type = SID_NAME_WKN_GRP;
+		goto ok;
+	}
+
+	/*
+	 * If we're told not to look up 'isolated' names then we're
+	 * done.
+	 */
+	if (!(flags & LOOKUP_NAME_ISOLATED)) {
+		TALLOC_FREE(tmp_ctx);
+		return false;
+	}
+
+	/*
+	 * No domain names beyond this point
+	 */
+	if (domain[0] != '\0') {
 		TALLOC_FREE(tmp_ctx);
 		return false;
 	}
@@ -146,6 +176,11 @@ bool lookup_name(TALLOC_CTX *mem_ctx,
 
 	/* 1. well-known names */
 
+	/*
+	 * Check for well known names without a domain name.
+	 * e.g. \Creator Owner.
+	 */
+
 	if ((flags & LOOKUP_NAME_WKN) &&
 	    lookup_wellknown_name(tmp_ctx, name, &sid, &domain))
 	{
@@ -215,8 +250,7 @@ bool lookup_name(TALLOC_CTX *mem_ctx,
 	    lookup_builtin_name(name, &rid))
 	{
 		domain = talloc_strdup(tmp_ctx, builtin_domain_name());
-		sid_copy(&sid, &global_sid_Builtin);
-		sid_append_rid(&sid, rid);
+		sid_compose(&sid, &global_sid_Builtin, rid);
 		type = SID_NAME_ALIAS;
 		goto ok;
 	}
@@ -230,8 +264,7 @@ bool lookup_name(TALLOC_CTX *mem_ctx,
 	    lookup_global_sam_name(name, flags, &rid, &type))
 	{
 		domain = talloc_strdup(tmp_ctx, get_global_sam_name());
-		sid_copy(&sid, get_global_sam_sid());
-		sid_append_rid(&sid, rid);
+		sid_compose(&sid, get_global_sam_sid(), rid);
 		goto ok;
 	}
 
@@ -257,10 +290,9 @@ bool lookup_name(TALLOC_CTX *mem_ctx,
 	 * that (yet), but give it a chance. */
 
 	if (IS_DC && winbind_lookup_name("", name, &sid, &type)) {
-		DOM_SID dom_sid;
-		uint32 tmp_rid;
+		struct dom_sid dom_sid;
 		enum lsa_SidType domain_type;
-		
+
 		if (type == SID_NAME_DOMAIN) {
 			/* Swap name and type */
 			tmp = name; name = domain; domain = tmp;
@@ -272,7 +304,7 @@ bool lookup_name(TALLOC_CTX *mem_ctx,
 		 * domain it figured out itself. Maybe fix that later... */
 
 		sid_copy(&dom_sid, &sid);
-		sid_split_rid(&dom_sid, &tmp_rid);
+		sid_split_rid(&dom_sid, NULL);
 
 		if (!winbind_lookup_sid(tmp_ctx, &dom_sid, &domain, NULL,
 					&domain_type) ||
@@ -290,7 +322,7 @@ bool lookup_name(TALLOC_CTX *mem_ctx,
 	/* 11. Ok, windows would end here. Samba has two more options:
                Unmapped users and unmapped groups */
 
-	if (((flags & LOOKUP_NAME_NO_NSS) == 0)
+	if (((flags & (LOOKUP_NAME_NO_NSS|LOOKUP_NAME_GROUP)) == 0)
 	    && lookup_unix_user_name(name, &sid)) {
 		domain = talloc_strdup(tmp_ctx, unix_users_domain_name());
 		type = SID_NAME_USER;
@@ -336,7 +368,10 @@ bool lookup_name(TALLOC_CTX *mem_ctx,
 			TALLOC_FREE(tmp_ctx);
 			return false;
 		}
-		strupper_m(tmp_dom);
+		if (!strupper_m(tmp_dom)) {
+			TALLOC_FREE(tmp_ctx);
+			return false;
+		}
 		*ret_domain = tmp_dom;
 	}
 
@@ -361,7 +396,7 @@ bool lookup_name(TALLOC_CTX *mem_ctx,
 bool lookup_name_smbconf(TALLOC_CTX *mem_ctx,
 		 const char *full_name, int flags,
 		 const char **ret_domain, const char **ret_name,
-		 DOM_SID *ret_sid, enum lsa_SidType *ret_type)
+		 struct dom_sid *ret_sid, enum lsa_SidType *ret_type)
 {
 	char *qualified_name;
 	const char *p;
@@ -389,6 +424,30 @@ bool lookup_name_smbconf(TALLOC_CTX *mem_ctx,
 				ret_sid, ret_type);
 	}
 
+	/* Try with winbind default domain name. */
+	if (lp_winbind_use_default_domain()) {
+		bool ok;
+
+		qualified_name = talloc_asprintf(mem_ctx,
+						 "%s\\%s",
+						 lp_workgroup(),
+						 full_name);
+		if (qualified_name == NULL) {
+			return false;
+		}
+
+		ok = lookup_name(mem_ctx,
+				 qualified_name,
+				 flags,
+				 ret_domain,
+				 ret_name,
+				 ret_sid,
+				 ret_type);
+		if (ok) {
+			return true;
+		}
+	}
+
 	/* Try with our own SAM name. */
 	qualified_name = talloc_asprintf(mem_ctx, "%s\\%s",
 				get_global_sam_name(),
@@ -402,7 +461,7 @@ bool lookup_name_smbconf(TALLOC_CTX *mem_ctx,
 				ret_sid, ret_type)) {
 		return true;
 	}
-	
+
 	/* Finally try with "Unix Users" or "Unix Group" */
 	qualified_name = talloc_asprintf(mem_ctx, "%s\\%s",
 				flags & LOOKUP_NAME_GROUP ?
@@ -419,8 +478,8 @@ bool lookup_name_smbconf(TALLOC_CTX *mem_ctx,
 }
 
 static bool wb_lookup_rids(TALLOC_CTX *mem_ctx,
-			   const DOM_SID *domain_sid,
-			   int num_rids, uint32 *rids,
+			   const struct dom_sid *domain_sid,
+			   int num_rids, uint32_t *rids,
 			   const char **domain_name,
 			   const char **names, enum lsa_SidType *types)
 {
@@ -469,7 +528,7 @@ static bool wb_lookup_rids(TALLOC_CTX *mem_ctx,
 	return true;
 }
 
-static bool lookup_rids(TALLOC_CTX *mem_ctx, const DOM_SID *domain_sid,
+static bool lookup_rids(TALLOC_CTX *mem_ctx, const struct dom_sid *domain_sid,
 			int num_rids, uint32_t *rids,
 			const char **domain_name,
 			const char ***names, enum lsa_SidType **types)
@@ -480,8 +539,8 @@ static bool lookup_rids(TALLOC_CTX *mem_ctx, const DOM_SID *domain_sid,
 		   sid_string_dbg(domain_sid)));
 
 	if (num_rids) {
-		*names = TALLOC_ZERO_ARRAY(mem_ctx, const char *, num_rids);
-		*types = TALLOC_ARRAY(mem_ctx, enum lsa_SidType, num_rids);
+		*names = talloc_zero_array(mem_ctx, const char *, num_rids);
+		*types = talloc_array(mem_ctx, enum lsa_SidType, num_rids);
 
 		if ((*names == NULL) || (*types == NULL)) {
 			return false;
@@ -494,7 +553,7 @@ static bool lookup_rids(TALLOC_CTX *mem_ctx, const DOM_SID *domain_sid,
 		*types = NULL;
 	}
 
-	if (sid_check_is_domain(domain_sid)) {
+	if (sid_check_is_our_sam(domain_sid)) {
 		NTSTATUS result;
 
 		if (*domain_name == NULL) {
@@ -543,9 +602,8 @@ static bool lookup_rids(TALLOC_CTX *mem_ctx, const DOM_SID *domain_sid,
 
 	if (sid_check_is_wellknown_domain(domain_sid, NULL)) {
 		for (i=0; i<num_rids; i++) {
-			DOM_SID sid;
-			sid_copy(&sid, domain_sid);
-			sid_append_rid(&sid, rids[i]);
+			struct dom_sid sid;
+			sid_compose(&sid, domain_sid, rids[i]);
 			if (lookup_wellknown_sid(mem_ctx, &sid,
 						 domain_name, &(*names)[i])) {
 				if ((*names)[i] == NULL) {
@@ -605,13 +663,13 @@ static bool lookup_rids(TALLOC_CTX *mem_ctx, const DOM_SID *domain_sid,
  * Is the SID a domain as such? If yes, lookup its name.
  */
 
-static bool lookup_as_domain(const DOM_SID *sid, TALLOC_CTX *mem_ctx,
+static bool lookup_as_domain(const struct dom_sid *sid, TALLOC_CTX *mem_ctx,
 			     const char **name)
 {
 	const char *tmp;
 	enum lsa_SidType type;
 
-	if (sid_check_is_domain(sid)) {
+	if (sid_check_is_our_sam(sid)) {
 		*name = talloc_strdup(mem_ctx, get_global_sam_name());
 		return true;
 	}
@@ -642,7 +700,7 @@ static bool lookup_as_domain(const DOM_SID *sid, TALLOC_CTX *mem_ctx,
 	}
 
 	if (IS_DC) {
-		uint32 i, num_domains;
+		uint32_t i, num_domains;
 		struct trustdom_info **domains;
 
 		/* This is relatively expensive, but it happens only on DCs
@@ -656,7 +714,7 @@ static bool lookup_as_domain(const DOM_SID *sid, TALLOC_CTX *mem_ctx,
 		}
 
 		for (i=0; i<num_domains; i++) {
-			if (sid_equal(sid, &domains[i]->sid)) {
+			if (dom_sid_equal(sid, &domains[i]->sid)) {
 				*name = talloc_strdup(mem_ctx,
 						      domains[i]->name);
 				return true;
@@ -693,7 +751,7 @@ static bool lookup_as_domain(const DOM_SID *sid, TALLOC_CTX *mem_ctx,
  * Level 6: Like 4
  */
 
-static bool check_dom_sid_to_level(const DOM_SID *sid, int level)
+static bool check_dom_sid_to_level(const struct dom_sid *sid, int level)
 {
 	int ret = false;
 
@@ -708,7 +766,7 @@ static bool check_dom_sid_to_level(const DOM_SID *sid, int level)
 	case 3:
 	case 4:
 	case 6:
-		ret = sid_check_is_domain(sid);
+		ret = sid_check_is_our_sam(sid);
 		break;
 	case 5:
 		ret = false;
@@ -728,13 +786,11 @@ static bool check_dom_sid_to_level(const DOM_SID *sid, int level)
  * This attempts to be as efficient as possible: It collects all SIDs
  * belonging to a domain and hands them in bulk to the appropriate lookup
  * function. In particular pdb_lookup_rids with ldapsam_trusted benefits
- * *hugely* from this. Winbind is going to be extended with a lookup_rids
- * interface as well, so on a DC we can do a bulk lsa_lookuprids to the
- * appropriate DC.
+ * *hugely* from this.
  */
 
 NTSTATUS lookup_sids(TALLOC_CTX *mem_ctx, int num_sids,
-		     const DOM_SID **sids, int level,
+		     const struct dom_sid **sids, int level,
 		     struct lsa_dom_info **ret_domains,
 		     struct lsa_name_info **ret_names)
 {
@@ -751,7 +807,7 @@ NTSTATUS lookup_sids(TALLOC_CTX *mem_ctx, int num_sids,
 	}
 
 	if (num_sids) {
-		name_infos = TALLOC_ARRAY(mem_ctx, struct lsa_name_info, num_sids);
+		name_infos = talloc_array(mem_ctx, struct lsa_name_info, num_sids);
 		if (name_infos == NULL) {
 			result = NT_STATUS_NO_MEMORY;
 			goto fail;
@@ -760,7 +816,7 @@ NTSTATUS lookup_sids(TALLOC_CTX *mem_ctx, int num_sids,
 		name_infos = NULL;
 	}
 
-	dom_infos = TALLOC_ZERO_ARRAY(mem_ctx, struct lsa_dom_info,
+	dom_infos = talloc_zero_array(mem_ctx, struct lsa_dom_info,
 				      LSA_REF_DOMAIN_LIST_MULTIPLIER);
 	if (dom_infos == NULL) {
 		result = NT_STATUS_NO_MEMORY;
@@ -782,8 +838,8 @@ NTSTATUS lookup_sids(TALLOC_CTX *mem_ctx, int num_sids,
 	 */
 
 	for (i=0; i<num_sids; i++) {
-		DOM_SID sid;
-		uint32 rid;
+		struct dom_sid sid;
+		uint32_t rid = 0;
 		const char *domain_name = NULL;
 
 		sid_copy(&sid, sids[i]);
@@ -801,7 +857,7 @@ NTSTATUS lookup_sids(TALLOC_CTX *mem_ctx, int num_sids,
 				result = NT_STATUS_NO_MEMORY;
 				goto fail;
 			}
-				
+
 			name_infos[i].rid = 0;
 			name_infos[i].type = SID_NAME_DOMAIN;
 			name_infos[i].name = NULL;
@@ -835,7 +891,7 @@ NTSTATUS lookup_sids(TALLOC_CTX *mem_ctx, int num_sids,
 			if (!dom_infos[j].valid) {
 				break;
 			}
-			if (sid_equal(&sid, &dom_infos[j].sid)) {
+			if (dom_sid_equal(&sid, &dom_infos[j].sid)) {
 				break;
 			}
 		}
@@ -896,13 +952,18 @@ NTSTATUS lookup_sids(TALLOC_CTX *mem_ctx, int num_sids,
 			break;
 		}
 
-		if (dom->num_idxs) {
-			if (!(rids = TALLOC_ARRAY(tmp_ctx, uint32, dom->num_idxs))) {
-				result = NT_STATUS_NO_MEMORY;
-				goto fail;
-			}
-		} else {
-			rids = NULL;
+		if (dom->num_idxs == 0) {
+			/*
+			 * This happens only if the only sid related to
+			 * this domain is the domain sid itself, which
+			 * is mapped to SID_NAME_DOMAIN above.
+			 */
+			continue;
+		}
+
+		if (!(rids = talloc_array(tmp_ctx, uint32_t, dom->num_idxs))) {
+			result = NT_STATUS_NO_MEMORY;
+			goto fail;
 		}
 
 		for (j=0; j<dom->num_idxs; j++) {
@@ -920,7 +981,7 @@ NTSTATUS lookup_sids(TALLOC_CTX *mem_ctx, int num_sids,
 			result = NT_STATUS_NO_MEMORY;
 			goto fail;
 		}
-			
+
 		for (j=0; j<dom->num_idxs; j++) {
 			int idx = dom->idxs[j];
 			name_infos[idx].type = types[j];
@@ -953,7 +1014,7 @@ NTSTATUS lookup_sids(TALLOC_CTX *mem_ctx, int num_sids,
  *THE CANONICAL* convert SID to name function.
 *****************************************************************/  
 
-bool lookup_sid(TALLOC_CTX *mem_ctx, const DOM_SID *sid,
+bool lookup_sid(TALLOC_CTX *mem_ctx, const struct dom_sid *sid,
 		const char **ret_domain, const char **ret_name,
 		enum lsa_SidType *ret_type)
 {
@@ -1013,128 +1074,23 @@ bool lookup_sid(TALLOC_CTX *mem_ctx, const DOM_SID *sid,
  modified to use linked lists by jra.
 *****************************************************************/  
 
-/*****************************************************************
-  Find a SID given a uid.
-*****************************************************************/
-
-static bool fetch_sid_from_uid_cache(DOM_SID *psid, uid_t uid)
-{
-	DATA_BLOB cache_value;
-
-	if (!memcache_lookup(NULL, UID_SID_CACHE,
-			     data_blob_const(&uid, sizeof(uid)),
-			     &cache_value)) {
-		return false;
-	}
-
-	memcpy(psid, cache_value.data, MIN(sizeof(*psid), cache_value.length));
-	SMB_ASSERT(cache_value.length >= offsetof(struct dom_sid, id_auth));
-	SMB_ASSERT(cache_value.length == ndr_size_dom_sid(psid, NULL, 0));
-
-	return true;
-}
-
-/*****************************************************************
-  Find a uid given a SID.
-*****************************************************************/
-
-static bool fetch_uid_from_cache( uid_t *puid, const DOM_SID *psid )
-{
-	DATA_BLOB cache_value;
-
-	if (!memcache_lookup(NULL, SID_UID_CACHE,
-			     data_blob_const(psid, ndr_size_dom_sid(psid, NULL, 0)),
-			     &cache_value)) {
-		return false;
-	}
-
-	SMB_ASSERT(cache_value.length == sizeof(*puid));
-	memcpy(puid, cache_value.data, sizeof(*puid));
-
-	return true;
-}
-
-/*****************************************************************
- Store uid to SID mapping in cache.
-*****************************************************************/
-
-void store_uid_sid_cache(const DOM_SID *psid, uid_t uid)
-{
-	memcache_add(NULL, SID_UID_CACHE,
-		     data_blob_const(psid, ndr_size_dom_sid(psid, NULL, 0)),
-		     data_blob_const(&uid, sizeof(uid)));
-	memcache_add(NULL, UID_SID_CACHE,
-		     data_blob_const(&uid, sizeof(uid)),
-		     data_blob_const(psid, ndr_size_dom_sid(psid, NULL, 0)));
-}
-
-/*****************************************************************
-  Find a SID given a gid.
-*****************************************************************/
-
-static bool fetch_sid_from_gid_cache(DOM_SID *psid, gid_t gid)
-{
-	DATA_BLOB cache_value;
-
-	if (!memcache_lookup(NULL, GID_SID_CACHE,
-			     data_blob_const(&gid, sizeof(gid)),
-			     &cache_value)) {
-		return false;
-	}
-
-	memcpy(psid, cache_value.data, MIN(sizeof(*psid), cache_value.length));
-	SMB_ASSERT(cache_value.length >= offsetof(struct dom_sid, id_auth));
-	SMB_ASSERT(cache_value.length == ndr_size_dom_sid(psid, NULL, 0));
-
-	return true;
-}
-
-/*****************************************************************
-  Find a gid given a SID.
-*****************************************************************/
-
-static bool fetch_gid_from_cache(gid_t *pgid, const DOM_SID *psid)
-{
-	DATA_BLOB cache_value;
-
-	if (!memcache_lookup(NULL, SID_GID_CACHE,
-			     data_blob_const(psid, ndr_size_dom_sid(psid, NULL, 0)),
-			     &cache_value)) {
-		return false;
-	}
-
-	SMB_ASSERT(cache_value.length == sizeof(*pgid));
-	memcpy(pgid, cache_value.data, sizeof(*pgid));
-
-	return true;
-}
-
-/*****************************************************************
- Store gid to SID mapping in cache.
-*****************************************************************/
-
-void store_gid_sid_cache(const DOM_SID *psid, gid_t gid)
-{
-	memcache_add(NULL, SID_GID_CACHE,
-		     data_blob_const(psid, ndr_size_dom_sid(psid, NULL, 0)),
-		     data_blob_const(&gid, sizeof(gid)));
-	memcache_add(NULL, GID_SID_CACHE,
-		     data_blob_const(&gid, sizeof(gid)),
-		     data_blob_const(psid, ndr_size_dom_sid(psid, NULL, 0)));
-}
 
 /*****************************************************************
  *THE LEGACY* convert uid_t to SID function.
 *****************************************************************/  
 
-static void legacy_uid_to_sid(DOM_SID *psid, uid_t uid)
+static void legacy_uid_to_sid(struct dom_sid *psid, uid_t uid)
 {
 	bool ret;
+	struct unixid id;
 
 	ZERO_STRUCTP(psid);
 
+	id.id = uid;
+	id.type = ID_TYPE_UID;
+
 	become_root();
-	ret = pdb_uid_to_sid(uid, psid);
+	ret = pdb_id_to_sid(&id, psid);
 	unbecome_root();
 
 	if (ret) {
@@ -1146,11 +1102,17 @@ static void legacy_uid_to_sid(DOM_SID *psid, uid_t uid)
 
 	uid_to_unix_users_sid(uid, psid);
 
+	{
+		struct unixid xid = {
+			.id = uid, .type = ID_TYPE_UID
+		};
+		idmap_cache_set_sid2unixid(psid, &xid);
+	}
+
  done:
 	DEBUG(10,("LEGACY: uid %u -> sid %s\n", (unsigned int)uid,
 		  sid_string_dbg(psid)));
 
-	store_uid_sid_cache(psid, uid);
 	return;
 }
 
@@ -1158,153 +1120,100 @@ static void legacy_uid_to_sid(DOM_SID *psid, uid_t uid)
  *THE LEGACY* convert gid_t to SID function.
 *****************************************************************/  
 
-static void legacy_gid_to_sid(DOM_SID *psid, gid_t gid)
+static void legacy_gid_to_sid(struct dom_sid *psid, gid_t gid)
 {
 	bool ret;
+	struct unixid id;
 
 	ZERO_STRUCTP(psid);
 
+	id.id = gid;
+	id.type = ID_TYPE_GID;
+
 	become_root();
-	ret = pdb_gid_to_sid(gid, psid);
+	ret = pdb_id_to_sid(&id, psid);
 	unbecome_root();
 
 	if (ret) {
 		/* This is a mapped group */
 		goto done;
 	}
-	
+
 	/* This is an unmapped group */
 
 	gid_to_unix_groups_sid(gid, psid);
 
+	{
+		struct unixid xid = {
+			.id = gid, .type = ID_TYPE_GID
+		};
+		idmap_cache_set_sid2unixid(psid, &xid);
+	}
+
  done:
 	DEBUG(10,("LEGACY: gid %u -> sid %s\n", (unsigned int)gid,
 		  sid_string_dbg(psid)));
 
-	store_gid_sid_cache(psid, gid);
 	return;
 }
 
 /*****************************************************************
- *THE LEGACY* convert SID to uid function.
+ *THE LEGACY* convert SID to id function.
 *****************************************************************/  
 
-static bool legacy_sid_to_uid(const DOM_SID *psid, uid_t *puid)
+static bool legacy_sid_to_unixid(const struct dom_sid *psid, struct unixid *id)
 {
-	enum lsa_SidType type;
-	uint32 rid;
+	bool ret;
 
-	if (sid_peek_check_rid(get_global_sam_sid(), psid, &rid)) {
-		union unid_t id;
-		bool ret;
+	become_root();
+	ret = pdb_sid_to_id(psid, id);
+	unbecome_root();
 
-		become_root();
-		ret = pdb_sid_to_id(psid, &id, &type);
-		unbecome_root();
-
-		if (ret) {
-			if (type != SID_NAME_USER) {
-				DEBUG(5, ("sid %s is a %s, expected a user\n",
-					  sid_string_dbg(psid),
-					  sid_type_lookup(type)));
-				return false;
-			}
-			*puid = id.uid;
-			goto done;
-		}
-
-		/* This was ours, but it was not mapped.  Fail */
-	}
-
-	DEBUG(10,("LEGACY: mapping failed for sid %s\n",
-		  sid_string_dbg(psid)));
-	return false;
-
-done:
-	DEBUG(10,("LEGACY: sid %s -> uid %u\n", sid_string_dbg(psid),
-		  (unsigned int)*puid ));
-
-	store_uid_sid_cache(psid, *puid);
-	return true;
-}
-
-/*****************************************************************
- *THE LEGACY* convert SID to gid function.
- Group mapping is used for gids that maps to Wellknown SIDs
-*****************************************************************/  
-
-static bool legacy_sid_to_gid(const DOM_SID *psid, gid_t *pgid)
-{
-	uint32 rid;
-	GROUP_MAP map;
-	union unid_t id;
-	enum lsa_SidType type;
-
-	if ((sid_check_is_in_builtin(psid) ||
-	     sid_check_is_in_wellknown_domain(psid))) {
-		bool ret;
-
-		become_root();
-		ret = pdb_getgrsid(&map, *psid);
-		unbecome_root();
-
-		if (ret) {
-			*pgid = map.gid;
-			goto done;
-		}
+	if (!ret) {
 		DEBUG(10,("LEGACY: mapping failed for sid %s\n",
 			  sid_string_dbg(psid)));
 		return false;
 	}
 
-	if (sid_peek_check_rid(get_global_sam_sid(), psid, &rid)) {
-		bool ret;
-
-		become_root();
-		ret = pdb_sid_to_id(psid, &id, &type);
-		unbecome_root();
-
-		if (ret) {
-			if ((type != SID_NAME_DOM_GRP) &&
-			    (type != SID_NAME_ALIAS)) {
-				DEBUG(5, ("LEGACY: sid %s is a %s, expected "
-					  "a group\n", sid_string_dbg(psid),
-					  sid_type_lookup(type)));
-				return false;
-			}
-			*pgid = id.gid;
-			goto done;
-		}
-	
-		/* This was ours, but it was not mapped.  Fail */
-	}
-
-	DEBUG(10,("LEGACY: mapping failed for sid %s\n",
-		  sid_string_dbg(psid)));
-	return false;
-	
- done:
-	DEBUG(10,("LEGACY: sid %s -> gid %u\n", sid_string_dbg(psid),
-		  (unsigned int)*pgid ));
-
-	store_gid_sid_cache(psid, *pgid);
-
 	return true;
 }
 
+static bool legacy_sid_to_gid(const struct dom_sid *psid, gid_t *pgid)
+{
+	struct unixid id;
+	if (!legacy_sid_to_unixid(psid, &id)) {
+		return false;
+	}
+	if (id.type == ID_TYPE_GID || id.type == ID_TYPE_BOTH) {
+		*pgid = id.id;
+		return true;
+	}
+	return false;
+}
+
+static bool legacy_sid_to_uid(const struct dom_sid *psid, uid_t *puid)
+{
+	struct unixid id;
+	if (!legacy_sid_to_unixid(psid, &id)) {
+		return false;
+	}
+	if (id.type == ID_TYPE_UID || id.type == ID_TYPE_BOTH) {
+		*puid = id.id;
+		return true;
+	}
+	return false;
+}
+
 /*****************************************************************
  *THE CANONICAL* convert uid_t to SID function.
 *****************************************************************/  
 
-void uid_to_sid(DOM_SID *psid, uid_t uid)
+void uid_to_sid(struct dom_sid *psid, uid_t uid)
 {
 	bool expired = true;
 	bool ret;
 	ZERO_STRUCTP(psid);
 
-	if (fetch_sid_from_uid_cache(psid, uid))
-		return;
-
 	/* Check the winbindd cache directly. */
 	ret = idmap_cache_find_uid2sid(uid, psid, &expired);
 
@@ -1341,7 +1250,6 @@ void uid_to_sid(DOM_SID *psid, uid_t uid)
 	DEBUG(10,("uid %u -> sid %s\n", (unsigned int)uid,
 		  sid_string_dbg(psid)));
 
-	store_uid_sid_cache(psid, uid);
 	return;
 }
 
@@ -1349,15 +1257,12 @@ void uid_to_sid(DOM_SID *psid, uid_t uid)
  *THE CANONICAL* convert gid_t to SID function.
 *****************************************************************/  
 
-void gid_to_sid(DOM_SID *psid, gid_t gid)
+void gid_to_sid(struct dom_sid *psid, gid_t gid)
 {
 	bool expired = true;
 	bool ret;
 	ZERO_STRUCTP(psid);
 
-	if (fetch_sid_from_gid_cache(psid, gid))
-		return;
-
 	/* Check the winbindd cache directly. */
 	ret = idmap_cache_find_gid2sid(gid, psid, &expired);
 
@@ -1394,27 +1299,134 @@ void gid_to_sid(DOM_SID *psid, gid_t gid)
 	DEBUG(10,("gid %u -> sid %s\n", (unsigned int)gid,
 		  sid_string_dbg(psid)));
 
-	store_gid_sid_cache(psid, gid);
 	return;
 }
 
+bool sids_to_unixids(const struct dom_sid *sids, uint32_t num_sids,
+		     struct unixid *ids)
+{
+	struct wbcDomainSid *wbc_sids = NULL;
+	struct wbcUnixId *wbc_ids = NULL;
+	uint32_t i, num_not_cached;
+	wbcErr err;
+	bool ret = false;
+
+	wbc_sids = talloc_array(talloc_tos(), struct wbcDomainSid, num_sids);
+	if (wbc_sids == NULL) {
+		return false;
+	}
+
+	num_not_cached = 0;
+
+	for (i=0; i<num_sids; i++) {
+		bool expired;
+		uint32_t rid;
+
+		if (sid_peek_check_rid(&global_sid_Unix_Users,
+				       &sids[i], &rid)) {
+			ids[i].type = ID_TYPE_UID;
+			ids[i].id = rid;
+			continue;
+		}
+		if (sid_peek_check_rid(&global_sid_Unix_Groups,
+				       &sids[i], &rid)) {
+			ids[i].type = ID_TYPE_GID;
+			ids[i].id = rid;
+			continue;
+		}
+		if (idmap_cache_find_sid2unixid(&sids[i], &ids[i], &expired)
+		    && !expired)
+		{
+			continue;
+		}
+		ids[i].type = ID_TYPE_NOT_SPECIFIED;
+		memcpy(&wbc_sids[num_not_cached], &sids[i],
+		       ndr_size_dom_sid(&sids[i], 0));
+		num_not_cached += 1;
+	}
+	if (num_not_cached == 0) {
+		goto done;
+	}
+	wbc_ids = talloc_array(talloc_tos(), struct wbcUnixId, num_not_cached);
+	if (wbc_ids == NULL) {
+		goto fail;
+	}
+	for (i=0; i<num_not_cached; i++) {
+		wbc_ids[i].type = WBC_ID_TYPE_NOT_SPECIFIED;
+	}
+	err = wbcSidsToUnixIds(wbc_sids, num_not_cached, wbc_ids);
+	if (!WBC_ERROR_IS_OK(err)) {
+		DEBUG(10, ("wbcSidsToUnixIds returned %s\n",
+			   wbcErrorString(err)));
+	}
+
+	num_not_cached = 0;
+
+	for (i=0; i<num_sids; i++) {
+		if (ids[i].type == ID_TYPE_NOT_SPECIFIED) {
+			switch (wbc_ids[num_not_cached].type) {
+			case WBC_ID_TYPE_UID:
+				ids[i].type = ID_TYPE_UID;
+				ids[i].id = wbc_ids[num_not_cached].id.uid;
+				break;
+			case WBC_ID_TYPE_GID:
+				ids[i].type = ID_TYPE_GID;
+				ids[i].id = wbc_ids[num_not_cached].id.gid;
+				break;
+			default:
+				/* The types match, and wbcUnixId -> id is a union anyway */
+				ids[i].type = (enum id_type)wbc_ids[num_not_cached].type;
+				ids[i].id = wbc_ids[num_not_cached].id.gid;
+				break;
+			}
+			num_not_cached += 1;
+		}
+	}
+
+	for (i=0; i<num_sids; i++) {
+		if (ids[i].type != ID_TYPE_NOT_SPECIFIED) {
+			continue;
+		}
+		if (legacy_sid_to_gid(&sids[i], &ids[i].id)) {
+			ids[i].type = ID_TYPE_GID;
+			continue;
+		}
+		if (legacy_sid_to_uid(&sids[i], &ids[i].id)) {
+			ids[i].type = ID_TYPE_UID;
+			continue;
+		}
+	}
+done:
+	for (i=0; i<num_sids; i++) {
+		switch(ids[i].type) {
+		case WBC_ID_TYPE_GID:
+		case WBC_ID_TYPE_UID:
+		case WBC_ID_TYPE_BOTH:
+			if (ids[i].id == -1) {
+				ids[i].type = ID_TYPE_NOT_SPECIFIED;
+			}
+			break;
+		case WBC_ID_TYPE_NOT_SPECIFIED:
+			break;
+		}
+	}
+
+	ret = true;
+fail:
+	TALLOC_FREE(wbc_ids);
+	TALLOC_FREE(wbc_sids);
+	return ret;
+}
+
 /*****************************************************************
  *THE CANONICAL* convert SID to uid function.
 *****************************************************************/  
 
-bool sid_to_uid(const DOM_SID *psid, uid_t *puid)
+bool sid_to_uid(const struct dom_sid *psid, uid_t *puid)
 {
 	bool expired = true;
 	bool ret;
-	uint32 rid;
-	gid_t gid;
-
-	if (fetch_uid_from_cache(puid, psid))
-		return true;
-
-	if (fetch_gid_from_cache(&gid, psid)) {
-		return false;
-	}
+	uint32_t rid;
 
 	/* Optimize for the Unix Users Domain
 	 * as the conversion is straightforward */
@@ -1455,7 +1467,6 @@ bool sid_to_uid(const DOM_SID *psid, uid_t *puid)
 	DEBUG(10,("sid %s -> uid %u\n", sid_string_dbg(psid),
 		(unsigned int)*puid ));
 
-	store_uid_sid_cache(psid, *puid);
 	return true;
 }
 
@@ -1464,18 +1475,11 @@ bool sid_to_uid(const DOM_SID *psid, uid_t *puid)
  Group mapping is used for gids that maps to Wellknown SIDs
 *****************************************************************/  
 
-bool sid_to_gid(const DOM_SID *psid, gid_t *pgid)
+bool sid_to_gid(const struct dom_sid *psid, gid_t *pgid)
 {
 	bool expired = true;
 	bool ret;
-	uint32 rid;
-	uid_t uid;
-
-	if (fetch_gid_from_cache(pgid, psid))
-		return true;
-
-	if (fetch_uid_from_cache(&uid, psid))
-		return false;
+	uint32_t rid;
 
 	/* Optimize for the Unix Groups Domain
 	 * as the conversion is straightforward */
@@ -1517,6 +1521,135 @@ bool sid_to_gid(const DOM_SID *psid, gid_t *pgid)
 	DEBUG(10,("sid %s -> gid %u\n", sid_string_dbg(psid),
 		  (unsigned int)*pgid ));
 
-	store_gid_sid_cache(psid, *pgid);
 	return true;
 }
+
+/**
+ * @brief This function gets the primary group SID mapping the primary
+ *        GID of the user as obtained by an actual getpwnam() call.
+ *        This is necessary to avoid issues with arbitrary group SIDs
+ *        stored in passdb. We try as hard as we can to get the SID
+ *        corresponding to the GID, including trying group mapping.
+ *        If nothing else works, we will force "Domain Users" as the
+ *        primary group.
+ *        This is needed because we must always be able to lookup the
+ *        primary group SID, so we cannot settle for an arbitrary SID.
+ *
+ *        This call can be expensive. Use with moderation.
+ *        If you have a "samu" struct around use pdb_get_group_sid()
+ *        instead as it does properly cache results.
+ *
+ * @param mem_ctx[in]     The memory context iused to allocate the result.
+ * @param username[in]    The user's name
+ * @param _pwd[in|out]    If available, pass in user's passwd struct.
+ *                        It will contain a tallocated passwd if NULL was
+ *                        passed in.
+ * @param _group_sid[out] The user's Primary Group SID
+ *
+ * @return NTSTATUS error code.
+ */
+NTSTATUS get_primary_group_sid(TALLOC_CTX *mem_ctx,
+				const char *username,
+				struct passwd **_pwd,
+				struct dom_sid **_group_sid)
+{
+	TALLOC_CTX *tmp_ctx;
+	bool need_lookup_sid = false;
+	struct dom_sid *group_sid;
+	struct passwd *pwd = *_pwd;
+
+	tmp_ctx = talloc_new(mem_ctx);
+	if (!tmp_ctx) {
+		return NT_STATUS_NO_MEMORY;
+	}
+
+	if (!pwd) {
+		pwd = Get_Pwnam_alloc(mem_ctx, username);
+		if (!pwd) {
+			DEBUG(0, ("Failed to find a Unix account for %s\n",
+				  username));
+			TALLOC_FREE(tmp_ctx);
+			return NT_STATUS_NO_SUCH_USER;
+		}
+	}
+
+	group_sid = talloc_zero(mem_ctx, struct dom_sid);
+	if (!group_sid) {
+		TALLOC_FREE(tmp_ctx);
+		return NT_STATUS_NO_MEMORY;
+	}
+
+	gid_to_sid(group_sid, pwd->pw_gid);
+	if (!is_null_sid(group_sid)) {
+		struct dom_sid domain_sid;
+		uint32_t rid;
+
+		/* We need a sid within our domain */
+		sid_copy(&domain_sid, group_sid);
+		sid_split_rid(&domain_sid, &rid);
+		if (dom_sid_equal(&domain_sid, get_global_sam_sid())) {
+			/*
+			 * As shortcut for the expensive lookup_sid call
+			 * compare the domain sid part
+			 */
+			switch (rid) {
+			case DOMAIN_RID_ADMINS:
+			case DOMAIN_RID_USERS:
+				goto done;
+			default:
+				need_lookup_sid = true;
+				break;
+			}
+		} else {
+			/* Try group mapping */
+			struct unixid id;
+
+			id.id = pwd->pw_gid;
+			id.type = ID_TYPE_GID;
+
+			ZERO_STRUCTP(group_sid);
+			if (pdb_id_to_sid(&id, group_sid)) {
+				need_lookup_sid = true;
+			}
+		}
+	}
+
+	/* We must verify that this is a valid SID that resolves to a
+	 * group of the correct type */
+	if (need_lookup_sid) {
+		enum lsa_SidType type = SID_NAME_UNKNOWN;
+		bool lookup_ret;
+
+		DEBUG(10, ("do lookup_sid(%s) for group of user %s\n",
+			   sid_string_dbg(group_sid), username));
+
+		/* Now check that it's actually a domain group and
+		 * not something else */
+		lookup_ret = lookup_sid(tmp_ctx, group_sid,
+					NULL, NULL, &type);
+
+		if (lookup_ret && (type == SID_NAME_DOM_GRP)) {
+			goto done;
+		}
+
+		DEBUG(3, ("Primary group %s for user %s is"
+			  " a %s and not a domain group\n",
+			  sid_string_dbg(group_sid), username,
+			  sid_type_lookup(type)));
+	}
+
+	/* Everything else, failed.
+	 * Just set it to the 'Domain Users' RID of 513 which will
+	   always resolve to a name */
+	DEBUG(3, ("Forcing Primary Group to 'Domain Users' for %s\n",
+		  username));
+
+	sid_compose(group_sid, get_global_sam_sid(), DOMAIN_RID_USERS);
+
+done:
+	*_pwd = talloc_move(mem_ctx, &pwd);
+	*_group_sid = talloc_move(mem_ctx, &group_sid);
+	TALLOC_FREE(tmp_ctx);
+	return NT_STATUS_OK;
+}
+
