@@ -29,7 +29,6 @@
 #include "messages.h"
 #include "lib/util/util_tdb.h"
 #include "librpc/gen_ndr/ndr_smbXsrv.h"
-#include <ccan/hash/hash.h>
 #include "serverid.h"
 
 struct smbXsrv_open_table {
@@ -49,7 +48,7 @@ static struct db_context *smbXsrv_open_global_db_ctx = NULL;
 
 NTSTATUS smbXsrv_open_global_init(void)
 {
-	const char *global_path = NULL;
+	char *global_path = NULL;
 	struct db_context *db_ctx = NULL;
 
 	if (smbXsrv_open_global_db_ctx != NULL) {
@@ -57,6 +56,9 @@ NTSTATUS smbXsrv_open_global_init(void)
 	}
 
 	global_path = lock_path("smbXsrv_open_global.tdb");
+	if (global_path == NULL) {
+		return NT_STATUS_NO_MEMORY;
+	}
 
 	db_ctx = db_open(NULL, global_path,
 			 0, /* hash_size */
@@ -64,7 +66,9 @@ NTSTATUS smbXsrv_open_global_init(void)
 			 TDB_CLEAR_IF_FIRST |
 			 TDB_INCOMPATIBLE_HASH,
 			 O_RDWR | O_CREAT, 0600,
-			 DBWRAP_LOCK_ORDER_1);
+			 DBWRAP_LOCK_ORDER_1,
+			 DBWRAP_FLAG_NONE);
+	TALLOC_FREE(global_path);
 	if (db_ctx == NULL) {
 		NTSTATUS status;
 
@@ -152,6 +156,7 @@ static NTSTATUS smbXsrv_open_table_init(struct smbXsrv_connection *conn,
 					uint32_t highest_id,
 					uint32_t max_opens)
 {
+	struct smbXsrv_client *client = conn->client;
 	struct smbXsrv_open_table *table;
 	NTSTATUS status;
 	uint64_t max_range;
@@ -168,7 +173,7 @@ static NTSTATUS smbXsrv_open_table_init(struct smbXsrv_connection *conn,
 		return NT_STATUS_INTERNAL_ERROR;
 	}
 
-	table = talloc_zero(conn, struct smbXsrv_open_table);
+	table = talloc_zero(client, struct smbXsrv_open_table);
 	if (table == NULL) {
 		return NT_STATUS_NO_MEMORY;
 	}
@@ -190,7 +195,7 @@ static NTSTATUS smbXsrv_open_table_init(struct smbXsrv_connection *conn,
 
 	table->global.db_ctx = smbXsrv_open_global_db_ctx;
 
-	conn->open_table = table;
+	client->open_table = table;
 	return NT_STATUS_OK;
 }
 
@@ -443,7 +448,9 @@ static NTSTATUS smbXsrv_open_local_lookup(struct smbXsrv_open_table *table,
 		return NT_STATUS_FILE_CLOSED;
 	}
 
-	state.op->idle_time = now;
+	if (now != 0) {
+		state.op->idle_time = now;
+	}
 
 	*_open = state.op;
 	return state.op->status;
@@ -577,6 +584,7 @@ static void smbXsrv_open_global_verify_record(struct db_record *db_rec,
 
 	val = dbwrap_record_get_value(db_rec);
 	if (val.dsize == 0) {
+		DEBUG(10, ("%s: empty value\n", __func__));
 		TALLOC_FREE(frame);
 		*is_free = true;
 		if (was_free) {
@@ -600,7 +608,7 @@ static void smbXsrv_open_global_verify_record(struct db_record *db_rec,
 	}
 
 	DEBUG(10,("smbXsrv_open_global_verify_record\n"));
-	if (DEBUGLVL(10)) {
+	if (CHECK_DEBUGLVL(10)) {
 		NDR_PRINT_DEBUG(smbXsrv_open_globalB, &global_blob);
 	}
 
@@ -622,11 +630,12 @@ static void smbXsrv_open_global_verify_record(struct db_record *db_rec,
 		exists = serverid_exists(&global->server_id);
 	}
 	if (!exists) {
+		struct server_id_buf idbuf;
 		DEBUG(2,("smbXsrv_open_global_verify_record: "
 			 "key '%s' server_id %s does not exist.\n",
 			 hex_encode_talloc(frame, key.dptr, key.dsize),
-			 server_id_str(frame, &global->server_id)));
-		if (DEBUGLVL(2)) {
+			 server_id_str_buf(global->server_id, &idbuf)));
+		if (CHECK_DEBUGLVL(2)) {
 			NDR_PRINT_DEBUG(smbXsrv_open_globalB, &global_blob);
 		}
 		TALLOC_FREE(frame);
@@ -692,7 +701,7 @@ static NTSTATUS smbXsrv_open_global_store(struct smbXsrv_open_global0 *global)
 		return status;
 	}
 
-	if (DEBUGLVL(10)) {
+	if (CHECK_DEBUGLVL(10)) {
 		DEBUG(10,("smbXsrv_open_global_store: key '%s' stored\n",
 			 hex_encode_talloc(global->db_rec, key.dptr, key.dsize)));
 		NDR_PRINT_DEBUG(smbXsrv_open_globalB, &global_blob);
@@ -737,6 +746,7 @@ static NTSTATUS smbXsrv_open_global_lookup(struct smbXsrv_open_table *table,
 					  mem_ctx,
 					  _global);
 	if (is_free) {
+		DEBUG(10, ("%s: is_free=true\n", __func__));
 		talloc_free(global_rec);
 		return NT_STATUS_OBJECT_NAME_NOT_FOUND;
 	}
@@ -769,7 +779,7 @@ NTSTATUS smbXsrv_open_create(struct smbXsrv_connection *conn,
 			     NTTIME now,
 			     struct smbXsrv_open **_open)
 {
-	struct smbXsrv_open_table *table = conn->open_table;
+	struct smbXsrv_open_table *table = conn->client->open_table;
 	struct db_record *local_rec = NULL;
 	struct smbXsrv_open *op = NULL;
 	void *ptr = NULL;
@@ -859,7 +869,7 @@ NTSTATUS smbXsrv_open_create(struct smbXsrv_connection *conn,
 		return status;
 	}
 
-	if (DEBUGLVL(10)) {
+	if (CHECK_DEBUGLVL(10)) {
 		struct smbXsrv_openB open_blob;
 
 		ZERO_STRUCT(open_blob);
@@ -879,12 +889,14 @@ uint32_t smbXsrv_open_hash(struct smbXsrv_open *_open)
 {
 	uint8_t buf[8+8+8];
 	uint32_t ret;
+	TDB_DATA key;
 
 	SBVAL(buf,  0, _open->global->open_persistent_id);
 	SBVAL(buf,  8, _open->global->open_volatile_id);
 	SBVAL(buf, 16, _open->global->open_time);
 
-	ret = hash(buf, sizeof(buf), 0);
+	key = (TDB_DATA) { .dptr = buf, .dsize = sizeof(buf) };
+	ret = tdb_jenkins_hash(&key);
 
 	if (ret == 0) {
 		ret = 1;
@@ -930,7 +942,7 @@ NTSTATUS smbXsrv_open_update(struct smbXsrv_open *op)
 		return status;
 	}
 
-	if (DEBUGLVL(10)) {
+	if (CHECK_DEBUGLVL(10)) {
 		struct smbXsrv_openB open_blob;
 
 		ZERO_STRUCT(open_blob);
@@ -1008,7 +1020,7 @@ NTSTATUS smbXsrv_open_close(struct smbXsrv_open *op, NTTIME now)
 			error = status;
 		}
 
-		if (NT_STATUS_IS_OK(status) && DEBUGLVL(10)) {
+		if (NT_STATUS_IS_OK(status) && CHECK_DEBUGLVL(10)) {
 			struct smbXsrv_openB open_blob;
 
 			ZERO_STRUCT(open_blob);
@@ -1098,7 +1110,7 @@ NTSTATUS smb1srv_open_table_init(struct smbXsrv_connection *conn)
 	 *
 	 * 0 and 0xFFFF are no valid ids.
 	 */
-	max_opens = conn->sconn->real_max_open_files;
+	max_opens = conn->client->sconn->real_max_open_files;
 	max_opens = MIN(max_opens, UINT16_MAX - 1);
 
 	return smbXsrv_open_table_init(conn, 1, UINT16_MAX - 1, max_opens);
@@ -1108,7 +1120,7 @@ NTSTATUS smb1srv_open_lookup(struct smbXsrv_connection *conn,
 			     uint16_t fnum, NTTIME now,
 			     struct smbXsrv_open **_open)
 {
-	struct smbXsrv_open_table *table = conn->open_table;
+	struct smbXsrv_open_table *table = conn->client->open_table;
 	uint32_t local_id = fnum;
 	uint32_t global_id = 0;
 
@@ -1132,7 +1144,7 @@ NTSTATUS smb2srv_open_table_init(struct smbXsrv_connection *conn)
 	 * transport connection (as we still have a 1:1 mapping
 	 * between process and transport connection).
 	 */
-	max_opens = conn->sconn->real_max_open_files;
+	max_opens = conn->client->sconn->real_max_open_files;
 	max_opens = MIN(max_opens, UINT16_MAX - 1);
 
 	return smbXsrv_open_table_init(conn, 1, UINT32_MAX - 1, max_opens);
@@ -1144,7 +1156,7 @@ NTSTATUS smb2srv_open_lookup(struct smbXsrv_connection *conn,
 			     NTTIME now,
 			     struct smbXsrv_open **_open)
 {
-	struct smbXsrv_open_table *table = conn->open_table;
+	struct smbXsrv_open_table *table = conn->client->open_table;
 	uint32_t local_id = volatile_id & UINT32_MAX;
 	uint64_t local_zeros = volatile_id & 0xFFFFFFFF00000000LLU;
 	uint32_t global_id = persistent_id & UINT32_MAX;
@@ -1168,11 +1180,11 @@ NTSTATUS smb2srv_open_lookup(struct smbXsrv_connection *conn,
 NTSTATUS smb2srv_open_recreate(struct smbXsrv_connection *conn,
 			       struct auth_session_info *session_info,
 			       uint64_t persistent_id,
-			       struct GUID create_guid,
+			       const struct GUID *create_guid,
 			       NTTIME now,
 			       struct smbXsrv_open **_open)
 {
-	struct smbXsrv_open_table *table = conn->open_table;
+	struct smbXsrv_open_table *table = conn->client->open_table;
 	struct db_record *local_rec = NULL;
 	struct smbXsrv_open *op = NULL;
 	void *ptr = NULL;
@@ -1183,15 +1195,18 @@ NTSTATUS smb2srv_open_recreate(struct smbXsrv_connection *conn,
 	struct security_token *current_token = NULL;
 
 	if (session_info == NULL) {
+		DEBUG(10, ("session_info=NULL\n"));
 		return NT_STATUS_INVALID_HANDLE;
 	}
 	current_token = session_info->security_token;
 
 	if (current_token == NULL) {
+		DEBUG(10, ("current_token=NULL\n"));
 		return NT_STATUS_INVALID_HANDLE;
 	}
 
 	if (global_zeros != 0) {
+		DEBUG(10, ("global_zeros!=0\n"));
 		return NT_STATUS_OBJECT_NAME_NOT_FOUND;
 	}
 
@@ -1204,10 +1219,20 @@ NTSTATUS smb2srv_open_recreate(struct smbXsrv_connection *conn,
 	status = smbXsrv_open_global_lookup(table, global_id, op, &op->global);
 	if (!NT_STATUS_IS_OK(status)) {
 		TALLOC_FREE(op);
+		DEBUG(10, ("smbXsrv_open_global_lookup returned %s\n",
+			   nt_errstr(status)));
 		return status;
 	}
 
-	if (!GUID_equal(&op->global->create_guid, &create_guid)) {
+	/*
+	 * If the provided create_guid is NULL, this means that
+	 * the reconnect request was a v1 request. In that case
+	 * we should skipt the create GUID verification, since
+	 * it is valid to v1-reconnect a v2-opened handle.
+	 */
+	if ((create_guid != NULL) &&
+	    !GUID_equal(&op->global->create_guid, create_guid))
+	{
 		TALLOC_FREE(op);
 		return NT_STATUS_OBJECT_NAME_NOT_FOUND;
 	}
@@ -1262,7 +1287,7 @@ NTSTATUS smb2srv_open_recreate(struct smbXsrv_connection *conn,
 		return status;
 	}
 
-	if (DEBUGLVL(10)) {
+	if (CHECK_DEBUGLVL(10)) {
 		struct smbXsrv_openB open_blob;
 
 		ZERO_STRUCT(open_blob);
@@ -1375,11 +1400,12 @@ NTSTATUS smbXsrv_open_global_traverse(
 
 NTSTATUS smbXsrv_open_cleanup(uint64_t persistent_id)
 {
-	NTSTATUS status;
+	NTSTATUS status = NT_STATUS_OK;
 	TALLOC_CTX *frame = talloc_stackframe();
 	struct smbXsrv_open_global0 *op = NULL;
 	uint8_t key_buf[SMBXSRV_OPEN_GLOBAL_TDB_KEY_SIZE];
 	TDB_DATA key;
+	TDB_DATA val;
 	struct db_record *rec;
 	bool delete_open = false;
 	uint32_t global_id = persistent_id & UINT32_MAX;
@@ -1395,6 +1421,14 @@ NTSTATUS smbXsrv_open_cleanup(uint64_t persistent_id)
 		goto done;
 	}
 
+	val = dbwrap_record_get_value(rec);
+	if (val.dsize == 0) {
+		DEBUG(10, ("smbXsrv_open_cleanup[global: 0x%08x] "
+			  "empty record in %s, skipping...\n",
+			   global_id, dbwrap_name(smbXsrv_open_global_db_ctx)));
+		goto done;
+	}
+
 	status = smbXsrv_open_global_parse_record(talloc_tos(), rec, &op);
 	if (!NT_STATUS_IS_OK(status)) {
 		DEBUG(1, ("smbXsrv_open_cleanup[global: 0x%08x] "
@@ -1420,9 +1454,11 @@ NTSTATUS smbXsrv_open_cleanup(uint64_t persistent_id)
 			   op->durable_timeout_msec / 1000,
 			   delete_open ? "" : " not"));
 	} else if (!serverid_exists(&op->server_id)) {
+		struct server_id_buf idbuf;
 		DEBUG(10, ("smbXsrv_open_cleanup[global: 0x%08x] "
 			   "server[%s] does not exist\n",
-			   global_id, server_id_str(frame, &op->server_id)));
+			   global_id,
+			   server_id_str_buf(op->server_id, &idbuf)));
 		delete_open = true;
 	}
 
