@@ -153,7 +153,7 @@ void nb_createx(const char *fname,
 	uint16_t fd = (uint16_t)-1;
 	int i;
 	NTSTATUS status;
-	uint32 desired_access;
+	uint32_t desired_access;
 
 	if (create_options & FILE_DIRECTORY_FILE) {
 		desired_access = FILE_READ_DATA;
@@ -166,7 +166,7 @@ void nb_createx(const char *fname,
 				0x0,
 				FILE_SHARE_READ|FILE_SHARE_WRITE, 
 				create_disposition, 
-				create_options, 0, &fd);
+				create_options, 0, &fd, NULL);
 	if (!NT_STATUS_IS_OK(status) && handle != -1) {
 		printf("ERROR: cli_ntcreate failed for %s - %s\n",
 		       fname, nt_errstr(status));
@@ -285,9 +285,9 @@ void nb_qfileinfo(int fnum)
 
 void nb_qfsinfo(int level)
 {
-	int bsize, total, avail;
+	uint64_t bsize, total, avail;
 	/* this is not the right call - we need cli_qfsinfo() */
-	cli_dskattr(c, &bsize, &total, &avail);
+	cli_disk_size(c, "", &bsize, &total, &avail);
 }
 
 static NTSTATUS find_fn(const char *mnt, struct file_info *finfo, const char *name,
@@ -306,7 +306,8 @@ void nb_flush(int fnum)
 {
 	int i;
 	i = find_handle(fnum);
-	/* hmmm, we don't have cli_flush() yet */
+
+	cli_flush(NULL, c, i);
 }
 
 static int total_deleted;
@@ -323,6 +324,7 @@ static NTSTATUS delete_fn(const char *mnt, struct file_info *finfo,
 	n = SMB_STRDUP(name);
 	n[strlen(n)-1] = 0;
 	if (asprintf(&s, "%s%s", n, finfo->name) == -1) {
+		free(n);
 		printf("asprintf failed\n");
 		return NT_STATUS_NO_MEMORY;
 	}
@@ -330,12 +332,15 @@ static NTSTATUS delete_fn(const char *mnt, struct file_info *finfo,
 		char *s2;
 		if (asprintf(&s2, "%s\\*", s) == -1) {
 			printf("asprintf failed\n");
+			free(s);
+			free(n);
 			return NT_STATUS_NO_MEMORY;
 		}
 		status = cli_list(c, s2, FILE_ATTRIBUTE_DIRECTORY, delete_fn, NULL);
+		free(s2);
 		if (!NT_STATUS_IS_OK(status)) {
+			free(s);
 			free(n);
-			free(s2);
 			return status;
 		}
 		nb_rmdir(s);
