@@ -370,10 +370,10 @@ static int net_getdomainsid(struct net_context *c, int argc, const char **argv)
 }
 
 static bool search_maxrid(struct pdb_search *search, const char *type,
-			  uint32 *max_rid)
+			  uint32_t *max_rid)
 {
 	struct samr_displayentry *entries;
-	uint32 i, num_entries;
+	uint32_t i, num_entries;
 
 	if (search == NULL) {
 		d_fprintf(stderr, _("get_maxrid: Could not search %s\n"), type);
@@ -387,9 +387,9 @@ static bool search_maxrid(struct pdb_search *search, const char *type,
 	return true;
 }
 
-static uint32 get_maxrid(void)
+static uint32_t get_maxrid(void)
 {
-	uint32 max_rid = 0;
+	uint32_t max_rid = 0;
 
 	if (!search_maxrid(pdb_search_users(talloc_tos(), 0), "users", &max_rid))
 		return 0;
@@ -407,7 +407,7 @@ static uint32 get_maxrid(void)
 
 static int net_maxrid(struct net_context *c, int argc, const char **argv)
 {
-	uint32 rid;
+	uint32_t rid;
 
 	if (argc != 0) {
 	        d_fprintf(stderr, "%s net maxrid\n", _("Usage:"));
@@ -743,6 +743,14 @@ static struct functable net_func[] = {
 		   "'net serverid' commands.")
 	},
 
+	{	"notify",
+		net_notify,
+		NET_TRANSPORT_LOCAL,
+		N_("notifyd client code"),
+		N_("  Use 'net help notify' to get more information about "
+		   "'net notify' commands.")
+	},
+
 #ifdef WITH_FAKE_KASERVER
 	{	"afs",
 		net_afs,
@@ -767,13 +775,14 @@ static struct functable net_func[] = {
 /****************************************************************************
   main program
 ****************************************************************************/
- int main(int argc, const char **argv)
+ int main(int argc, char **argv)
 {
 	int opt,i;
 	char *p;
 	int rc = 0;
 	int argc_new = 0;
 	const char ** argv_new;
+	const char **argv_const = discard_const_p(const char *, argv);
 	poptContext pc;
 	TALLOC_CTX *frame = talloc_stackframe();
 	struct net_context *c = talloc_zero(frame, struct net_context);
@@ -838,7 +847,7 @@ static struct functable net_func[] = {
 
 	setup_logging(argv[0], DEBUG_STDERR);
 
-	load_case_tables();
+	smb_init_locale();
 
 	setlocale(LC_ALL, "");
 #if defined(HAVE_BINDTEXTDOMAIN)
@@ -852,7 +861,7 @@ static struct functable net_func[] = {
 	lp_set_cmdline("log level", "0");
 	c->private_data = net_func;
 
-	pc = poptGetContext(NULL, argc, (const char **) argv, long_options,
+	pc = poptGetContext(NULL, argc, argv_const, long_options,
 			    POPT_CONTEXT_KEEP_FIRST);
 
 	while((opt = poptGetNextOpt(pc)) != -1) {
@@ -883,12 +892,28 @@ static struct functable net_func[] = {
 		default:
 			d_fprintf(stderr, _("\nInvalid option %s: %s\n"),
 				 poptBadOption(pc, 0), poptStrerror(opt));
-			net_help(c, argc, argv);
+			net_help(c, argc, argv_const);
 			exit(1);
 		}
 	}
 
-	lp_load_global(get_dyn_CONFIGFILE());
+	if (!lp_load_initial_only(get_dyn_CONFIGFILE())) {
+		d_fprintf(stderr, "Can't load %s - run testparm to debug it\n",
+			  get_dyn_CONFIGFILE());
+		exit(1);
+	}
+
+	/*
+	 * Failing to init the msg_ctx isn't a fatal error. Only root-level
+	 * things (joining/leaving domains etc.) will be denied.
+	 */
+	c->msg_ctx = messaging_init(c, samba_tevent_context_init(c));
+
+	if (!lp_load_global(get_dyn_CONFIGFILE())) {
+		d_fprintf(stderr, "Can't load %s - run testparm to debug it\n",
+			  get_dyn_CONFIGFILE());
+		exit(1);
+	}
 
 #if defined(HAVE_BIND_TEXTDOMAIN_CODESET)
 	/* Bind our gettext results to 'unix charset'
@@ -952,11 +977,6 @@ static struct functable net_func[] = {
 
 	popt_burn_cmdline_password(argc, argv);
 
-	/* Failing to init the msg_ctx isn't a fatal error. Only
-	   root-level things (joining/leaving domains etc.) will be denied. */
-
-	c->msg_ctx = messaging_init(c, samba_tevent_context_init(c));
-
 	rc = net_run_function(c, argc_new-1, argv_new+1, "net", net_func);
 
 	DEBUG(2,("return code = %d\n", rc));
