@@ -19,7 +19,10 @@
 */
 
 #include "includes.h"
+#include "system/passwd.h"
+#include "system/filesys.h"
 #include "utils/net.h"
+#include "../libcli/security/security.h"
 
 struct {
 	const char *us_errstr;
@@ -37,6 +40,8 @@ struct {
 	{N_("Path not allowed"), USERSHARE_PATH_NOT_ALLOWED},
 	{N_("Path is not a directory"), USERSHARE_PATH_NOT_DIRECTORY},
 	{N_("System error"), USERSHARE_POSIX_ERR},
+	{N_("Malformed sharename definition"), USERSHARE_MALFORMED_SHARENAME_DEF},
+	{N_("Bad sharename (doesn't match filename)"), USERSHARE_BAD_SHARENAME},
 	{NULL,(enum usershare_err)-1}
 };
 
@@ -130,7 +135,7 @@ int net_usershare_usage(struct net_context *c, int argc, const char **argv)
 
 static char *get_basepath(TALLOC_CTX *ctx)
 {
-	char *basepath = talloc_strdup(ctx, lp_usershare_path());
+	char *basepath = lp_usershare_path(ctx);
 
 	if (!basepath) {
 		return NULL;
@@ -169,7 +174,7 @@ static int net_usershare_delete(struct net_context *c, int argc, const char **ar
 
 	us_path = talloc_asprintf(talloc_tos(),
 				"%s/%s",
-				lp_usershare_path(),
+				lp_usershare_path(talloc_tos()),
 				sharename);
 	if (!us_path) {
 		TALLOC_FREE(sharename);
@@ -203,8 +208,8 @@ static struct file_list *flist;
 
 static int get_share_list(TALLOC_CTX *ctx, const char *wcard, bool only_ours)
 {
-	SMB_STRUCT_DIR *dp;
-	SMB_STRUCT_DIRENT *de;
+	DIR *dp;
+	struct dirent *de;
 	uid_t myuid = geteuid();
 	struct file_list *fl = NULL;
 	char *basepath = get_basepath(ctx);
@@ -212,7 +217,7 @@ static int get_share_list(TALLOC_CTX *ctx, const char *wcard, bool only_ours)
 	if (!basepath) {
 		return -1;
 	}
-	dp = sys_opendir(basepath);
+	dp = opendir(basepath);
 	if (!dp) {
 		d_fprintf(stderr,
 			_("get_share_list: cannot open usershare directory %s. "
@@ -221,7 +226,7 @@ static int get_share_list(TALLOC_CTX *ctx, const char *wcard, bool only_ours)
 		return -1;
 	}
 
-	while((de = sys_readdir(dp)) != 0) {
+	while((de = readdir(dp)) != 0) {
 		SMB_STRUCT_STAT sbuf;
 		char *path;
 		const char *n = de->d_name;
@@ -244,7 +249,7 @@ static int get_share_list(TALLOC_CTX *ctx, const char *wcard, bool only_ours)
 					basepath,
 					n);
 		if (!path) {
-			sys_closedir(dp);
+			closedir(dp);
 			return -1;
 		}
 
@@ -273,21 +278,21 @@ static int get_share_list(TALLOC_CTX *ctx, const char *wcard, bool only_ours)
 		}
 
 		/* (Finally) - add to list. */
-		fl = TALLOC_P(ctx, struct file_list);
+		fl = talloc(ctx, struct file_list);
 		if (!fl) {
-			sys_closedir(dp);
+			closedir(dp);
 			return -1;
 		}
 		fl->pathname = talloc_strdup(ctx, n);
 		if (!fl->pathname) {
-			sys_closedir(dp);
+			closedir(dp);
 			return -1;
 		}
 
 		DLIST_ADD(flist, fl);
 	}
 
-	sys_closedir(dp);
+	closedir(dp);
 	return 0;
 }
 
@@ -328,10 +333,11 @@ static int info_fn(struct file_list *fl, void *priv)
 	struct net_context *c = pi->c;
 	int fd = -1;
 	int numlines = 0;
-	SEC_DESC *psd = NULL;
+	struct security_descriptor *psd = NULL;
 	char *basepath;
 	char *sharepath = NULL;
 	char *comment = NULL;
+	char *cp_sharename = NULL;
 	char *acl_str;
 	int num_aces;
 	char sep_str[2];
@@ -353,9 +359,9 @@ static int info_fn(struct file_list *fl, void *priv)
 	}
 
 #ifdef O_NOFOLLOW
-	fd = sys_open(basepath, O_RDONLY|O_NOFOLLOW, 0);
+	fd = open(basepath, O_RDONLY|O_NOFOLLOW, 0);
 #else
-	fd = sys_open(basepath, O_RDONLY, 0);
+	fd = open(basepath, O_RDONLY, 0);
 #endif
 
 	if (fd == -1) {
@@ -392,6 +398,7 @@ static int info_fn(struct file_list *fl, void *priv)
 	us_err = parse_usershare_file(ctx, &sbuf, fl->pathname, -1, lines, numlines,
 				&sharepath,
 				&comment,
+				&cp_sharename,
 				&psd,
 				&guest_ok);
 
@@ -473,13 +480,13 @@ static int info_fn(struct file_list *fl, void *priv)
 
 	/* NOTE: This is smb.conf-like output. Do not translate. */
 	if (pi->op == US_INFO_OP) {
-		d_printf("[%s]\n", fl->pathname );
+		d_printf("[%s]\n", cp_sharename );
 		d_printf("path=%s\n", sharepath );
 		d_printf("comment=%s\n", comment);
 		d_printf("%s\n", acl_str);
 		d_printf("guest_ok=%c\n\n", guest_ok ? 'y' : 'n');
 	} else if (pi->op == US_LIST_OP) {
-		d_printf("%s\n", fl->pathname);
+		d_printf("%s\n", cp_sharename);
 	}
 
 	return 0;
@@ -516,7 +523,9 @@ static int net_usershare_info(struct net_context *c, int argc, const char **argv
 			return net_usershare_info_usage(c, argc, argv);
 	}
 
-	strlower_m(wcard);
+	if (!strlower_m(wcard)) {
+		return -1;
+	}
 
 	ctx = talloc_init("share_info");
 	ret = get_share_list(ctx, wcard, only_ours);
@@ -539,8 +548,8 @@ static int net_usershare_info(struct net_context *c, int argc, const char **argv
 
 static int count_num_usershares(void)
 {
-	SMB_STRUCT_DIR *dp;
-	SMB_STRUCT_DIRENT *de;
+	DIR *dp;
+	struct dirent *de;
 	int num_usershares = 0;
 	TALLOC_CTX *ctx = talloc_tos();
 	char *basepath = get_basepath(ctx);
@@ -549,7 +558,7 @@ static int count_num_usershares(void)
 		return -1;
 	}
 
-	dp = sys_opendir(basepath);
+	dp = opendir(basepath);
 	if (!dp) {
 		d_fprintf(stderr,
 			_("count_num_usershares: cannot open usershare "
@@ -558,7 +567,7 @@ static int count_num_usershares(void)
 		return -1;
 	}
 
-	while((de = sys_readdir(dp)) != 0) {
+	while((de = readdir(dp)) != 0) {
 		SMB_STRUCT_STAT sbuf;
 		char *path;
 		const char *n = de->d_name;
@@ -581,7 +590,7 @@ static int count_num_usershares(void)
 				basepath,
 				n);
 		if (!path) {
-			sys_closedir(dp);
+			closedir(dp);
 			return -1;
 		}
 
@@ -603,7 +612,7 @@ static int count_num_usershares(void)
 		num_usershares++;
 	}
 
-	sys_closedir(dp);
+	closedir(dp);
 	return num_usershares;
 }
 
@@ -617,6 +626,7 @@ static int net_usershare_add(struct net_context *c, int argc, const char **argv)
 	SMB_STRUCT_STAT sbuf;
 	SMB_STRUCT_STAT lsbuf;
 	char *sharename;
+	const char *cp_sharename;
 	char *full_path;
 	char *full_path_tmp;
 	const char *us_path;
@@ -632,6 +642,7 @@ static int net_usershare_add(struct net_context *c, int argc, const char **argv)
 	uid_t myeuid = geteuid();
 	bool guest_ok = false;
 	int num_usershares;
+	mode_t mask;
 
 	us_comment = "";
 	arg_acl = "S-1-1-0:R";
@@ -645,21 +656,25 @@ static int net_usershare_add(struct net_context *c, int argc, const char **argv)
 		default:
 			return net_usershare_add_usage(c, argc, argv);
 		case 2:
+			cp_sharename = argv[0];
 			sharename = strlower_talloc(ctx, argv[0]);
 			us_path = argv[1];
 			break;
 		case 3:
+			cp_sharename = argv[0];
 			sharename = strlower_talloc(ctx, argv[0]);
 			us_path = argv[1];
 			us_comment = argv[2];
 			break;
 		case 4:
+			cp_sharename = argv[0];
 			sharename = strlower_talloc(ctx, argv[0]);
 			us_path = argv[1];
 			us_comment = argv[2];
 			arg_acl = argv[3];
 			break;
 		case 5:
+			cp_sharename = argv[0];
 			sharename = strlower_talloc(ctx, argv[0]);
 			us_path = argv[1];
 			us_comment = argv[2];
@@ -797,7 +812,7 @@ static int net_usershare_add(struct net_context *c, int argc, const char **argv)
 	num_aces += count_chars(pacl,',');
 
 	for (i = 0; i < num_aces; i++) {
-		DOM_SID sid;
+		struct dom_sid sid;
 		const char *pcolon = strchr_m(pacl, ':');
 		const char *name;
 
@@ -881,7 +896,9 @@ static int net_usershare_add(struct net_context *c, int argc, const char **argv)
 	}
 
 	/* Create a temporary filename for this share. */
+	mask = umask(S_IRWXO | S_IRWXG);
 	tmpfd = mkstemp(full_path_tmp);
+	umask(mask);
 
 	if (tmpfd == -1) {
 		d_fprintf(stderr,
@@ -897,6 +914,7 @@ static int net_usershare_add(struct net_context *c, int argc, const char **argv)
 			  _("net usershare add: cannot lstat tmp file %s\n"),
 			  full_path_tmp );
 		TALLOC_FREE(ctx);
+		close(tmpfd);
 		return -1;
 	}
 
@@ -906,6 +924,7 @@ static int net_usershare_add(struct net_context *c, int argc, const char **argv)
 			  _("net usershare add: cannot fstat tmp file %s\n"),
 			  full_path_tmp );
 		TALLOC_FREE(ctx);
+		close(tmpfd);
 		return -1;
 	}
 
@@ -915,6 +934,7 @@ static int net_usershare_add(struct net_context *c, int argc, const char **argv)
 			    "file ?\n"),
 			  full_path_tmp );
 		TALLOC_FREE(ctx);
+		close(tmpfd);
 		return -1;
 	}
 
@@ -924,13 +944,20 @@ static int net_usershare_add(struct net_context *c, int argc, const char **argv)
 			    "to 0644n"),
 			  full_path_tmp );
 		TALLOC_FREE(ctx);
+		close(tmpfd);
 		return -1;
 	}
 
 	/* Create the in-memory image of the file. */
 	file_img = talloc_strdup(ctx, "#VERSION 2\npath=");
-	file_img = talloc_asprintf_append(file_img, "%s\ncomment=%s\nusershare_acl=%s\nguest_ok=%c\n",
-			us_path, us_comment, us_acl, guest_ok ? 'y' : 'n');
+	file_img = talloc_asprintf_append(file_img,
+			"%s\ncomment=%s\nusershare_acl=%s\n"
+			"guest_ok=%c\nsharename=%s\n",
+			us_path,
+			us_comment,
+			us_acl,
+			guest_ok ? 'y' : 'n',
+			cp_sharename);
 
 	to_write = strlen(file_img);
 
@@ -941,6 +968,7 @@ static int net_usershare_add(struct net_context *c, int argc, const char **argv)
 			(unsigned int)to_write, full_path_tmp, strerror(errno));
 		unlink(full_path_tmp);
 		TALLOC_FREE(ctx);
+		close(tmpfd);
 		return -1;
 	}
 
@@ -1013,7 +1041,9 @@ static int net_usershare_list(struct net_context *c, int argc,
 			return net_usershare_list_usage(c, argc, argv);
 	}
 
-	strlower_m(wcard);
+	if (!strlower_m(wcard)) {
+		return -1;
+	}
 
 	ctx = talloc_init("share_list");
 	ret = get_share_list(ctx, wcard, only_ours);
@@ -1036,7 +1066,7 @@ static int net_usershare_list(struct net_context *c, int argc,
 
 int net_usershare(struct net_context *c, int argc, const char **argv)
 {
-	SMB_STRUCT_DIR *dp;
+	DIR *dp;
 
 	struct functable func[] = {
 		{
@@ -1081,13 +1111,13 @@ int net_usershare(struct net_context *c, int argc, const char **argv)
 		return -1;
 	}
 
-	dp = sys_opendir(lp_usershare_path());
+	dp = opendir(lp_usershare_path(talloc_tos()));
 	if (!dp) {
 		int err = errno;
 		d_fprintf(stderr,
 			_("net usershare: cannot open usershare directory %s. "
 			  "Error %s\n"),
-			lp_usershare_path(), strerror(err) );
+			lp_usershare_path(talloc_tos()), strerror(err) );
 		if (err == EACCES) {
 			d_fprintf(stderr,
 				_("You do not have permission to create a "
@@ -1100,7 +1130,7 @@ int net_usershare(struct net_context *c, int argc, const char **argv)
 		}
 		return -1;
 	}
-	sys_closedir(dp);
+	closedir(dp);
 
 	return net_run_function(c, argc, argv, "net usershare", func);
 }
