@@ -29,16 +29,32 @@
 #include "lib/events/events.h"
 #include "libcli/smb2/smb2.h"
 #include "libcli/smb2/smb2_calls.h"
+#include "libcli/smb/smbXcli_base.h"
 #include "librpc/rpc/dcerpc.h"
 #include "librpc/rpc/dcerpc_proto.h"
 #include "auth/credentials/credentials.h"
 #include "param/param.h"
 #include "libcli/resolve/resolve.h"
 
+struct dcerpc_pipe_connect {
+	struct dcecli_connection *conn;
+	struct dcerpc_binding *binding;
+	const struct ndr_interface_table *interface;
+	struct cli_credentials *creds;
+	struct resolve_context *resolve_ctx;
+	struct {
+		const char *dir;
+	} ncalrpc;
+	struct {
+		struct smbXcli_conn *conn;
+		struct smbXcli_session *session;
+		struct smbXcli_tcon *tcon;
+		const char *pipe_name;
+	} smb;
+};
 
 struct pipe_np_smb_state {
 	struct smb_composite_connect conn;
-	struct smbcli_tree *tree;
 	struct dcerpc_pipe_connect io;
 };
 
@@ -58,28 +74,49 @@ static void continue_pipe_open_smb(struct composite_context *ctx)
 	composite_done(c);
 }
 
+static void continue_smb_open(struct composite_context *c);
 
 /*
   Stage 2 of ncacn_np_smb: Open a named pipe after successful smb connection
 */
 static void continue_smb_connect(struct composite_context *ctx)
 {
-	struct composite_context *open_ctx;
 	struct composite_context *c = talloc_get_type(ctx->async.private_data,
 						      struct composite_context);
 	struct pipe_np_smb_state *s = talloc_get_type(c->private_data,
 						      struct pipe_np_smb_state);
-	
+	struct smbcli_tree *t;
+
 	/* receive result of smb connect request */
-	c->status = smb_composite_connect_recv(ctx, c);
+	c->status = smb_composite_connect_recv(ctx, s->io.conn);
 	if (!composite_is_ok(c)) return;
 
+	t = s->conn.out.tree;
+
 	/* prepare named pipe open parameters */
-	s->tree         = s->conn.out.tree;
-	s->io.pipe_name = s->io.binding->endpoint;
+	s->io.smb.conn = t->session->transport->conn;
+	s->io.smb.session = t->session->smbXcli;
+	s->io.smb.tcon = t->smbXcli;
+	smb1cli_tcon_set_id(s->io.smb.tcon, t->tid);
+	s->io.smb.pipe_name = dcerpc_binding_get_string_option(s->io.binding,
+							       "endpoint");
+
+	continue_smb_open(c);
+}
+
+static void continue_smb_open(struct composite_context *c)
+{
+	struct pipe_np_smb_state *s = talloc_get_type(c->private_data,
+						      struct pipe_np_smb_state);
+	struct composite_context *open_ctx;
 
 	/* send named pipe open request */
-	open_ctx = dcerpc_pipe_open_smb_send(s->io.pipe, s->tree, s->io.pipe_name);
+	open_ctx = dcerpc_pipe_open_smb_send(s->io.conn,
+					     s->io.smb.conn,
+					     s->io.smb.session,
+					     s->io.smb.tcon,
+					     DCERPC_REQUEST_TIMEOUT * 1000,
+					     s->io.smb.pipe_name);
 	if (composite_nomem(open_ctx, c)) return;
 
 	composite_continue(c, open_ctx, continue_pipe_open_smb, c);
@@ -96,9 +133,10 @@ static struct composite_context *dcerpc_pipe_connect_ncacn_np_smb_send(TALLOC_CT
 	struct pipe_np_smb_state *s;
 	struct composite_context *conn_req;
 	struct smb_composite_connect *conn;
+	uint32_t flags;
 
 	/* composite context allocation and setup */
-	c = composite_create(mem_ctx, io->pipe->conn->event_ctx);
+	c = composite_create(mem_ctx, io->conn->event_ctx);
 	if (c == NULL) return NULL;
 
 	s = talloc_zero(c, struct pipe_np_smb_state);
@@ -108,14 +146,20 @@ static struct composite_context *dcerpc_pipe_connect_ncacn_np_smb_send(TALLOC_CT
 	s->io  = *io;
 	conn   = &s->conn;
 
+	if (smbXcli_conn_is_connected(s->io.smb.conn)) {
+		continue_smb_open(c);
+		return c;
+	}
+
 	/* prepare smb connection parameters: we're connecting to IPC$ share on
 	   remote rpc server */
-	conn->in.dest_host              = s->io.binding->host;
-	conn->in.dest_ports                  = lpcfg_smb_ports(lp_ctx);
-	if (s->io.binding->target_hostname == NULL)
-		conn->in.called_name = "*SMBSERVER"; /* FIXME: This is invalid */
-	else
-		conn->in.called_name            = s->io.binding->target_hostname;
+	conn->in.dest_host = dcerpc_binding_get_string_option(s->io.binding, "host");
+	conn->in.dest_ports = lpcfg_smb_ports(lp_ctx);
+	conn->in.called_name =
+		dcerpc_binding_get_string_option(s->io.binding, "target_hostname");
+	if (conn->in.called_name == NULL) {
+		conn->in.called_name = "*SMBSERVER";
+	}
 	conn->in.socket_options         = lpcfg_socket_options(lp_ctx);
 	conn->in.service                = "IPC$";
 	conn->in.service_type           = NULL;
@@ -132,16 +176,22 @@ static struct composite_context *dcerpc_pipe_connect_ncacn_np_smb_send(TALLOC_CT
 	 * setup) or if asked to do so by the caller (perhaps a SAMR password change?)
 	 */
 	s->conn.in.credentials = s->io.creds;
-	if (s->io.binding->flags & (DCERPC_SCHANNEL|DCERPC_ANON_FALLBACK)) {
+	flags = dcerpc_binding_get_flags(s->io.binding);
+	if (flags & (DCERPC_SCHANNEL|DCERPC_ANON_FALLBACK)) {
 		conn->in.fallback_to_anonymous  = true;
 	} else {
 		conn->in.fallback_to_anonymous  = false;
 	}
 
+	conn->in.options.min_protocol = PROTOCOL_NT1;
+	conn->in.options.max_protocol = PROTOCOL_NT1;
+
+	conn->in.options.signing = lpcfg_client_ipc_signing(lp_ctx);
+
 	/* send smb connect request */
-	conn_req = smb_composite_connect_send(conn, s->io.pipe->conn, 
+	conn_req = smb_composite_connect_send(conn, s->io.conn,
 					      s->io.resolve_ctx,
-					      s->io.pipe->conn->event_ctx);
+					      c->event_ctx);
 	if (composite_nomem(conn_req, c)) return c;
 
 	composite_continue(c, conn_req, continue_smb_connect, c);
@@ -160,54 +210,30 @@ static NTSTATUS dcerpc_pipe_connect_ncacn_np_smb_recv(struct composite_context *
 	return status;
 }
 
-
-struct pipe_np_smb2_state {
-	struct smb2_tree *tree;
-	struct dcerpc_pipe_connect io;
-};
-
-
-/*
-  Stage 3 of ncacn_np_smb: Named pipe opened (or not)
-*/
-static void continue_pipe_open_smb2(struct composite_context *ctx)
-{
-	struct composite_context *c = talloc_get_type(ctx->async.private_data,
-						      struct composite_context);
-
-	/* receive result of named pipe open request on smb2 */
-	c->status = dcerpc_pipe_open_smb2_recv(ctx);
-	if (!composite_is_ok(c)) return;
-
-	composite_done(c);
-}
-
-
 /*
   Stage 2 of ncacn_np_smb2: Open a named pipe after successful smb2 connection
 */
 static void continue_smb2_connect(struct tevent_req *subreq)
 {
-	struct composite_context *open_req;
 	struct composite_context *c =
 		tevent_req_callback_data(subreq,
 		struct composite_context);
-	struct pipe_np_smb2_state *s = talloc_get_type(c->private_data,
-						       struct pipe_np_smb2_state);
+	struct pipe_np_smb_state *s = talloc_get_type(c->private_data,
+						      struct pipe_np_smb_state);
+	struct smb2_tree *t;
 
 	/* receive result of smb2 connect request */
-	c->status = smb2_connect_recv(subreq, c, &s->tree);
+	c->status = smb2_connect_recv(subreq, s->io.conn, &t);
 	TALLOC_FREE(subreq);
 	if (!composite_is_ok(c)) return;
 
-	/* prepare named pipe open parameters */
-	s->io.pipe_name = s->io.binding->endpoint;
+	s->io.smb.conn = t->session->transport->conn;
+	s->io.smb.session = t->session->smbXcli;
+	s->io.smb.tcon = t->smbXcli;
+	s->io.smb.pipe_name = dcerpc_binding_get_string_option(s->io.binding,
+							       "endpoint");
 
-	/* send named pipe open request */
-	open_req = dcerpc_pipe_open_smb2_send(s->io.pipe, s->tree, s->io.pipe_name);
-	if (composite_nomem(open_req, c)) return;
-
-	composite_continue(c, open_req, continue_pipe_open_smb2, c);
+	continue_smb_open(c);
 }
 
 
@@ -221,36 +247,55 @@ static struct composite_context *dcerpc_pipe_connect_ncacn_np_smb2_send(
 					struct loadparm_context *lp_ctx)
 {
 	struct composite_context *c;
-	struct pipe_np_smb2_state *s;
+	struct pipe_np_smb_state *s;
 	struct tevent_req *subreq;
 	struct smbcli_options options;
+	const char *host;
+	uint32_t flags;
 
 	/* composite context allocation and setup */
-	c = composite_create(mem_ctx, io->pipe->conn->event_ctx);
+	c = composite_create(mem_ctx, io->conn->event_ctx);
 	if (c == NULL) return NULL;
 
-	s = talloc_zero(c, struct pipe_np_smb2_state);
+	s = talloc_zero(c, struct pipe_np_smb_state);
 	if (composite_nomem(s, c)) return c;
 	c->private_data = s;
 
 	s->io = *io;
 
+	if (smbXcli_conn_is_connected(s->io.smb.conn)) {
+		continue_smb_open(c);
+		return c;
+	}
+
+	host = dcerpc_binding_get_string_option(s->io.binding, "host");
+	flags = dcerpc_binding_get_flags(s->io.binding);
+
 	/*
 	 * provide proper credentials - user supplied or anonymous in case this is
 	 * schannel connection
 	 */
-	if (s->io.binding->flags & DCERPC_SCHANNEL) {
-		s->io.creds = cli_credentials_init(mem_ctx);
+	if (flags & DCERPC_SCHANNEL) {
+		s->io.creds = cli_credentials_init_anon(mem_ctx);
 		if (composite_nomem(s->io.creds, c)) return c;
-
-		cli_credentials_guess(s->io.creds, lp_ctx);
 	}
 
 	lpcfg_smbcli_options(lp_ctx, &options);
 
+	options.min_protocol = lpcfg_client_ipc_min_protocol(lp_ctx);
+	if (options.min_protocol < PROTOCOL_SMB2_02) {
+		options.min_protocol = PROTOCOL_SMB2_02;
+	}
+	options.max_protocol = lpcfg_client_ipc_max_protocol(lp_ctx);
+	if (options.max_protocol < PROTOCOL_SMB2_02) {
+		options.max_protocol = PROTOCOL_SMB2_02;
+	}
+
+	options.signing = lpcfg_client_ipc_signing(lp_ctx);
+
 	/* send smb2 connect request */
 	subreq = smb2_connect_send(s, c->event_ctx,
-			s->io.binding->host,
+			host,
 			lpcfg_parm_string_list(mem_ctx, lp_ctx, NULL, "smb2", "ports", NULL),
 			"IPC$",
 			s->io.resolve_ctx,
@@ -293,9 +338,23 @@ static void continue_pipe_open_ncacn_ip_tcp(struct composite_context *ctx)
 {
 	struct composite_context *c = talloc_get_type(ctx->async.private_data,
 						      struct composite_context);
+	struct pipe_ip_tcp_state *s = talloc_get_type(c->private_data,
+						      struct pipe_ip_tcp_state);
+	char *localaddr = NULL;
+	char *remoteaddr = NULL;
 
 	/* receive result of named pipe open request on tcp/ip */
-	c->status = dcerpc_pipe_open_tcp_recv(ctx);
+	c->status = dcerpc_pipe_open_tcp_recv(ctx, s, &localaddr, &remoteaddr);
+	if (!composite_is_ok(c)) return;
+
+	c->status = dcerpc_binding_set_string_option(s->io.binding,
+						     "localaddress",
+						     localaddr);
+	if (!composite_is_ok(c)) return;
+
+	c->status = dcerpc_binding_set_string_option(s->io.binding,
+						     "host",
+						     remoteaddr);
 	if (!composite_is_ok(c)) return;
 
 	composite_done(c);
@@ -312,9 +371,10 @@ static struct composite_context* dcerpc_pipe_connect_ncacn_ip_tcp_send(TALLOC_CT
 	struct composite_context *c;
 	struct pipe_ip_tcp_state *s;
 	struct composite_context *pipe_req;
+	const char *endpoint;
 
 	/* composite context allocation and setup */
-	c = composite_create(mem_ctx, io->pipe->conn->event_ctx);
+	c = composite_create(mem_ctx, io->conn->event_ctx);
 	if (c == NULL) return NULL;
 
 	s = talloc_zero(c, struct pipe_ip_tcp_state);
@@ -322,15 +382,25 @@ static struct composite_context* dcerpc_pipe_connect_ncacn_ip_tcp_send(TALLOC_CT
 	c->private_data = s;
 
 	/* store input parameters in state structure */
-	s->io               = *io;
-	s->localaddr        = talloc_reference(c, io->binding->localaddress);
-	s->host             = talloc_reference(c, io->binding->host);
-	s->target_hostname  = talloc_reference(c, io->binding->target_hostname);
-                             /* port number is a binding endpoint here */
-	s->port             = atoi(io->binding->endpoint);   
+	s->io = *io;
+	s->localaddr = dcerpc_binding_get_string_option(io->binding,
+							"localaddress");
+	s->host = dcerpc_binding_get_string_option(io->binding, "host");
+	s->target_hostname = dcerpc_binding_get_string_option(io->binding,
+							      "target_hostname");
+	endpoint = dcerpc_binding_get_string_option(io->binding, "endpoint");
+	/* port number is a binding endpoint here */
+	if (endpoint != NULL) {
+		s->port = atoi(endpoint);
+	}
+
+	if (s->port == 0) {
+		composite_error(c, NT_STATUS_INVALID_PARAMETER_MIX);
+		return c;
+	}
 
 	/* send pipe open request on tcp/ip */
-	pipe_req = dcerpc_pipe_open_tcp_send(s->io.pipe->conn, s->localaddr, s->host, s->target_hostname,
+	pipe_req = dcerpc_pipe_open_tcp_send(s->io.conn, s->localaddr, s->host, s->target_hostname,
 					     s->port, io->resolve_ctx);
 	composite_continue(c, pipe_req, continue_pipe_open_ncacn_ip_tcp, c);
 	return c;
@@ -349,6 +419,177 @@ static NTSTATUS dcerpc_pipe_connect_ncacn_ip_tcp_recv(struct composite_context *
 }
 
 
+struct pipe_http_state {
+	struct dcerpc_pipe_connect io;
+	const char *localaddr;
+	const char *rpc_server;
+	uint32_t rpc_server_port;
+	char *rpc_proxy;
+	uint32_t rpc_proxy_port;
+	char *http_proxy;
+	uint32_t http_proxy_port;
+	bool use_tls;
+	bool use_proxy;
+	bool use_ntlm;
+	struct loadparm_context *lp_ctx;
+};
+
+/*
+  Stage 2 of ncacn_http: rpc pipe opened (or not)
+ */
+static void continue_pipe_open_ncacn_http(struct tevent_req *subreq)
+{
+	struct composite_context *c = NULL;
+	struct pipe_http_state *s = NULL;
+	struct tstream_context *stream = NULL;
+	struct tevent_queue *queue = NULL;
+
+	c = tevent_req_callback_data(subreq, struct composite_context);
+	s = talloc_get_type(c->private_data, struct pipe_http_state);
+
+	/* receive result of RoH connect request */
+	c->status = dcerpc_pipe_open_roh_recv(subreq, s->io.conn,
+	                                      &stream, &queue);
+	TALLOC_FREE(subreq);
+	if (!composite_is_ok(c)) return;
+
+	s->io.conn->transport.transport = NCACN_HTTP;
+	s->io.conn->transport.stream = stream;
+	s->io.conn->transport.write_queue = queue;
+	s->io.conn->transport.pending_reads = 0;
+
+	composite_done(c);
+}
+
+/*
+  Initiate async open of a rpc connection to a rpc pipe using HTTP transport,
+  and using the binding structure to determine the endpoint and options
+*/
+static struct composite_context* dcerpc_pipe_connect_ncacn_http_send(
+		TALLOC_CTX *mem_ctx, struct dcerpc_pipe_connect *io,
+		struct loadparm_context *lp_ctx)
+{
+	struct composite_context *c;
+	struct pipe_http_state *s;
+	struct tevent_req *subreq;
+	const char *endpoint;
+	const char *use_proxy;
+	char *proxy;
+	char *port;
+	const char *opt;
+
+	/* composite context allocation and setup */
+	c = composite_create(mem_ctx, io->conn->event_ctx);
+	if (c == NULL) return NULL;
+
+	s = talloc_zero(c, struct pipe_http_state);
+	if (composite_nomem(s, c)) return c;
+	c->private_data = s;
+
+	/* store input parameters in state structure */
+	s->lp_ctx	= lp_ctx;
+	s->io		= *io;
+	s->localaddr 	= dcerpc_binding_get_string_option(io->binding,
+							"localaddress");
+	/* RPC server and port (the endpoint) */
+	s->rpc_server = dcerpc_binding_get_string_option(io->binding, "host");
+	endpoint = dcerpc_binding_get_string_option(io->binding, "endpoint");
+	if (endpoint == NULL) {
+		composite_error(c, NT_STATUS_INVALID_PARAMETER_MIX);
+		return c;
+	}
+	s->rpc_server_port = atoi(endpoint);
+	if (s->rpc_server_port == 0) {
+		composite_error(c, NT_STATUS_INVALID_PARAMETER_MIX);
+		return c;
+	}
+
+	/* Use TLS */
+	opt = dcerpc_binding_get_string_option(io->binding, "HttpUseTls");
+	if (opt) {
+		if (strcasecmp(opt, "true") == 0) {
+			s->use_tls = true;
+		} else if (strcasecmp(opt, "false") == 0) {
+			s->use_tls = false;
+		} else {
+			composite_error(c, NT_STATUS_INVALID_PARAMETER_MIX);
+			return c;
+		}
+	} else {
+		s->use_tls = true;
+	}
+
+	/* RPC Proxy */
+	proxy = port = talloc_strdup(s, dcerpc_binding_get_string_option(
+			io->binding, "RpcProxy"));
+	s->rpc_proxy  = strsep(&port, ":");
+	if (proxy && port) {
+		s->rpc_proxy_port = atoi(port);
+	} else {
+		s->rpc_proxy_port = s->use_tls ? 443 : 80;
+	}
+	if (s->rpc_proxy == NULL) {
+		s->rpc_proxy = talloc_strdup(s, s->rpc_server);
+		if (composite_nomem(s->rpc_proxy, c)) return c;
+	}
+
+	/* HTTP Proxy */
+	proxy = port = talloc_strdup(s, dcerpc_binding_get_string_option(
+			io->binding, "HttpProxy"));
+	s->http_proxy = strsep(&port, ":");
+	if (proxy && port) {
+		s->http_proxy_port = atoi(port);
+	} else {
+		s->http_proxy_port = s->use_tls ? 443 : 80;
+	}
+
+	/* Use local proxy */
+	use_proxy = dcerpc_binding_get_string_option(io->binding,
+	                                         "HttpConnectOption");
+	if (use_proxy && strcasecmp(use_proxy, "UseHttpProxy")) {
+		s->use_proxy = true;
+	}
+
+	/* If use local proxy set, the http proxy should be provided */
+	if (s->use_proxy && !s->http_proxy) {
+		composite_error(c, NT_STATUS_INVALID_PARAMETER_MIX);
+		return c;
+	}
+
+	/* Check which HTTP authentication method to use */
+	opt = dcerpc_binding_get_string_option(io->binding, "HttpAuthOption");
+	if (opt) {
+		if (strcasecmp(opt, "basic") == 0) {
+			s->use_ntlm = false;
+		} else if (strcasecmp(opt, "ntlm") == 0) {
+			s->use_ntlm = true;
+		} else {
+			composite_error(c, NT_STATUS_INVALID_PARAMETER_MIX);
+			return c;
+		}
+	} else {
+		s->use_ntlm = true;
+	}
+
+	subreq = dcerpc_pipe_open_roh_send(s->io.conn, s->localaddr,
+	                                   s->rpc_server, s->rpc_server_port,
+	                                   s->rpc_proxy, s->rpc_proxy_port,
+	                                   s->http_proxy, s->http_proxy_port,
+	                                   s->use_tls, s->use_proxy,
+	                                   s->io.creds, io->resolve_ctx,
+	                                   s->lp_ctx, s->use_ntlm);
+	if (composite_nomem(subreq, c)) return c;
+
+	tevent_req_set_callback(subreq, continue_pipe_open_ncacn_http, c);
+	return c;
+}
+
+static NTSTATUS dcerpc_pipe_connect_ncacn_http_recv(struct composite_context *c)
+{
+	return composite_wait_free(c);
+}
+
+
 struct pipe_unix_state {
 	struct dcerpc_pipe_connect io;
 	const char *path;
@@ -383,7 +624,7 @@ static struct composite_context* dcerpc_pipe_connect_ncacn_unix_stream_send(TALL
 	struct composite_context *pipe_req;
 
 	/* composite context allocation and setup */
-	c = composite_create(mem_ctx, io->pipe->conn->event_ctx);
+	c = composite_create(mem_ctx, io->conn->event_ctx);
 	if (c == NULL) return NULL;
 
 	s = talloc_zero(c, struct pipe_unix_state);
@@ -393,18 +634,15 @@ static struct composite_context* dcerpc_pipe_connect_ncacn_unix_stream_send(TALL
 	/* prepare pipe open parameters and store them in state structure
 	   also, verify whether biding endpoint is not null */
 	s->io = *io;
-	
-	if (!io->binding->endpoint) {
-		DEBUG(0, ("Path to unix socket not specified\n"));
-		composite_error(c, NT_STATUS_INVALID_PARAMETER);
+
+	s->path = dcerpc_binding_get_string_option(io->binding, "endpoint");
+	if (s->path == NULL) {
+		composite_error(c, NT_STATUS_INVALID_PARAMETER_MIX);
 		return c;
 	}
 
-	s->path  = talloc_strdup(c, io->binding->endpoint);  /* path is a binding endpoint here */
-	if (composite_nomem(s->path, c)) return c;
-
 	/* send pipe open request on unix socket */
-	pipe_req = dcerpc_pipe_open_unix_stream_send(s->io.pipe->conn, s->path);
+	pipe_req = dcerpc_pipe_open_unix_stream_send(s->io.conn, s->path);
 	composite_continue(c, pipe_req, continue_pipe_open_ncacn_unix_stream, c);
 	return c;
 }
@@ -449,14 +687,15 @@ static void continue_pipe_open_ncalrpc(struct composite_context *ctx)
    the binding structure to determine the endpoint and options
 */
 static struct composite_context* dcerpc_pipe_connect_ncalrpc_send(TALLOC_CTX *mem_ctx,
-								  struct dcerpc_pipe_connect *io, struct loadparm_context *lp_ctx)
+								  struct dcerpc_pipe_connect *io)
 {
 	struct composite_context *c;
 	struct pipe_ncalrpc_state *s;
 	struct composite_context *pipe_req;
+	const char *endpoint;
 
 	/* composite context allocation and setup */
-	c = composite_create(mem_ctx, io->pipe->conn->event_ctx);
+	c = composite_create(mem_ctx, io->conn->event_ctx);
 	if (c == NULL) return NULL;
 
 	s = talloc_zero(c, struct pipe_ncalrpc_state);
@@ -466,9 +705,16 @@ static struct composite_context* dcerpc_pipe_connect_ncalrpc_send(TALLOC_CTX *me
 	/* store input parameters in state structure */
 	s->io  = *io;
 
+	endpoint = dcerpc_binding_get_string_option(io->binding, "endpoint");
+	if (endpoint == NULL) {
+		composite_error(c, NT_STATUS_INVALID_PARAMETER_MIX);
+		return c;
+	}
+
 	/* send pipe open request */
-	pipe_req = dcerpc_pipe_open_pipe_send(s->io.pipe->conn, lpcfg_ncalrpc_dir(lp_ctx),
-					      s->io.binding->endpoint);
+	pipe_req = dcerpc_pipe_open_pipe_send(s->io.conn,
+					      s->io.ncalrpc.dir,
+					      endpoint);
 	composite_continue(c, pipe_req, continue_pipe_open_ncalrpc, c);
 	return c;
 }
@@ -500,6 +746,7 @@ static void continue_connect(struct composite_context *c, struct pipe_connect_st
 static void continue_pipe_connect_ncacn_np_smb2(struct composite_context *ctx);
 static void continue_pipe_connect_ncacn_np_smb(struct composite_context *ctx);
 static void continue_pipe_connect_ncacn_ip_tcp(struct composite_context *ctx);
+static void continue_pipe_connect_ncacn_http(struct composite_context *ctx);
 static void continue_pipe_connect_ncacn_unix(struct composite_context *ctx);
 static void continue_pipe_connect_ncalrpc(struct composite_context *ctx);
 static void continue_pipe_connect(struct composite_context *c, struct pipe_connect_state *s);
@@ -515,12 +762,14 @@ static void continue_map_binding(struct composite_context *ctx)
 						      struct composite_context);
 	struct pipe_connect_state *s = talloc_get_type(c->private_data,
 						       struct pipe_connect_state);
-	
+	const char *endpoint;
+
 	c->status = dcerpc_epm_map_binding_recv(ctx);
 	if (!composite_is_ok(c)) return;
 
-	DEBUG(4,("Mapped to DCERPC endpoint %s\n", s->binding->endpoint));
-	
+	endpoint = dcerpc_binding_get_string_option(s->binding, "endpoint");
+	DEBUG(4,("Mapped to DCERPC endpoint %s\n", endpoint));
+
 	continue_connect(c, s);
 }
 
@@ -536,21 +785,33 @@ static void continue_connect(struct composite_context *c, struct pipe_connect_st
 	struct composite_context *ncacn_np_smb2_req;
 	struct composite_context *ncacn_np_smb_req;
 	struct composite_context *ncacn_ip_tcp_req;
+	struct composite_context *ncacn_http_req;
 	struct composite_context *ncacn_unix_req;
 	struct composite_context *ncalrpc_req;
+	enum dcerpc_transport_t transport;
+	enum protocol_types min_ipc_protocol;
+	uint32_t flags;
 
 	/* dcerpc pipe connect input parameters */
-	pc.pipe         = s->pipe;
+	ZERO_STRUCT(pc);
+	pc.conn         = s->pipe->conn;
 	pc.binding      = s->binding;
-	pc.pipe_name    = NULL;
 	pc.interface    = s->table;
 	pc.creds        = s->credentials;
 	pc.resolve_ctx  = lpcfg_resolve_context(s->lp_ctx);
 
+	transport = dcerpc_binding_get_transport(s->binding);
+	flags = dcerpc_binding_get_flags(s->binding);
+
+	min_ipc_protocol = lpcfg_client_ipc_min_protocol(s->lp_ctx);
+	if (min_ipc_protocol >= PROTOCOL_SMB2_02) {
+		flags |= DCERPC_SMB2;
+	}
+
 	/* connect dcerpc pipe depending on required transport */
-	switch (s->binding->transport) {
+	switch (transport) {
 	case NCACN_NP:
-		if (pc.binding->flags & DCERPC_SMB2) {
+		if (flags & DCERPC_SMB2) {
 			/* new varient of SMB a.k.a. SMB2 */
 			ncacn_np_smb2_req = dcerpc_pipe_connect_ncacn_np_smb2_send(c, &pc, s->lp_ctx);
 			composite_continue(c, ncacn_np_smb2_req, continue_pipe_connect_ncacn_np_smb2, c);
@@ -569,13 +830,22 @@ static void continue_connect(struct composite_context *c, struct pipe_connect_st
 		composite_continue(c, ncacn_ip_tcp_req, continue_pipe_connect_ncacn_ip_tcp, c);
 		return;
 
+	case NCACN_HTTP:
+		ncacn_http_req = dcerpc_pipe_connect_ncacn_http_send(c, &pc, s->lp_ctx);
+		composite_continue(c, ncacn_http_req, continue_pipe_connect_ncacn_http, c);
+		return;
+
 	case NCACN_UNIX_STREAM:
 		ncacn_unix_req = dcerpc_pipe_connect_ncacn_unix_stream_send(c, &pc);
 		composite_continue(c, ncacn_unix_req, continue_pipe_connect_ncacn_unix, c);
 		return;
 
 	case NCALRPC:
-		ncalrpc_req = dcerpc_pipe_connect_ncalrpc_send(c, &pc, s->lp_ctx);
+		pc.ncalrpc.dir = lpcfg_ncalrpc_dir(s->lp_ctx);
+		c->status = dcerpc_binding_set_string_option(s->binding, "ncalrpc_dir",
+							     pc.ncalrpc.dir);
+		if (!composite_is_ok(c)) return;
+		ncalrpc_req = dcerpc_pipe_connect_ncalrpc_send(c, &pc);
 		composite_continue(c, ncalrpc_req, continue_pipe_connect_ncalrpc, c);
 		return;
 
@@ -639,6 +909,23 @@ static void continue_pipe_connect_ncacn_ip_tcp(struct composite_context *ctx)
 }
 
 
+/*
+  Stage 3 of pipe_connect_b: Receive result of pipe connect request on http
+*/
+static void continue_pipe_connect_ncacn_http(struct composite_context *ctx)
+{
+	struct composite_context *c = talloc_get_type(ctx->async.private_data,
+						      struct composite_context);
+	struct pipe_connect_state *s = talloc_get_type(c->private_data,
+						       struct pipe_connect_state);
+
+	c->status = dcerpc_pipe_connect_ncacn_http_recv(ctx);
+	if (!composite_is_ok(c)) return;
+
+	continue_pipe_connect(c, s);
+}
+
+
 /*
   Stage 3 of pipe_connect_b: Receive result of pipe connect request on unix socket
 */
@@ -681,9 +968,8 @@ static void continue_pipe_connect(struct composite_context *c, struct pipe_conne
 {
 	struct composite_context *auth_bind_req;
 
-	s->pipe->binding = s->binding;
-	if (!talloc_reference(s->pipe, s->binding)) {
-		composite_error(c, NT_STATUS_NO_MEMORY);
+	s->pipe->binding = dcerpc_binding_dup(s->pipe, s->binding);
+	if (composite_nomem(s->pipe->binding, c)) {
 		return;
 	}
 
@@ -731,7 +1017,7 @@ static void dcerpc_connect_timeout_handler(struct tevent_context *ev, struct tev
   specified binding structure to determine the endpoint and options
 */
 _PUBLIC_ struct composite_context* dcerpc_pipe_connect_b_send(TALLOC_CTX *parent_ctx,
-						     struct dcerpc_binding *binding,
+						     const struct dcerpc_binding *binding,
 						     const struct ndr_interface_table *table,
 						     struct cli_credentials *credentials,
 						     struct tevent_context *ev,
@@ -739,6 +1025,9 @@ _PUBLIC_ struct composite_context* dcerpc_pipe_connect_b_send(TALLOC_CTX *parent
 {
 	struct composite_context *c;
 	struct pipe_connect_state *s;
+	enum dcerpc_transport_t transport;
+	const char *endpoint = NULL;
+	struct cli_credentials *epm_creds = NULL;
 
 	/* composite context allocation and setup */
 	c = composite_create(parent_ctx, ev);
@@ -755,10 +1044,11 @@ _PUBLIC_ struct composite_context* dcerpc_pipe_connect_b_send(TALLOC_CTX *parent
 	if (composite_nomem(s->pipe, c)) return c;
 
 	if (DEBUGLEVEL >= 10)
-		s->pipe->conn->packet_log_dir = lpcfg_lockdir(lp_ctx);
+		s->pipe->conn->packet_log_dir = lpcfg_lock_directory(lp_ctx);
 
 	/* store parameters in state structure */
-	s->binding      = binding;
+	s->binding      = dcerpc_binding_dup(s, binding);
+	if (composite_nomem(s->binding, c)) return c;
 	s->table        = table;
 	s->credentials  = credentials;
 	s->lp_ctx 	= lp_ctx;
@@ -769,31 +1059,37 @@ _PUBLIC_ struct composite_context* dcerpc_pipe_connect_b_send(TALLOC_CTX *parent
 	tevent_add_timer(c->event_ctx, c,
 			 timeval_current_ofs(DCERPC_REQUEST_TIMEOUT, 0),
 			 dcerpc_connect_timeout_handler, c);
-	
-	switch (s->binding->transport) {
-	case NCA_UNKNOWN: {
+
+	transport = dcerpc_binding_get_transport(s->binding);
+
+	switch (transport) {
+	case NCACN_NP:
+	case NCACN_IP_TCP:
+	case NCALRPC:
+		endpoint = dcerpc_binding_get_string_option(s->binding, "endpoint");
+
+		/* anonymous credentials for rpc connection used to get endpoint mapping */
+		epm_creds = cli_credentials_init_anon(s);
+		if (composite_nomem(epm_creds, c)) return c;
+
+		break;
+	case NCACN_HTTP:
+		endpoint = dcerpc_binding_get_string_option(s->binding, "endpoint");
+		epm_creds = credentials;
+		break;
+	default:
+		break;
+	}
+
+	if (endpoint == NULL) {
 		struct composite_context *binding_req;
+
 		binding_req = dcerpc_epm_map_binding_send(c, s->binding, s->table,
+							  epm_creds,
 							  s->pipe->conn->event_ctx,
 							  s->lp_ctx);
 		composite_continue(c, binding_req, continue_map_binding, c);
 		return c;
-		}
-
-	case NCACN_NP:
-	case NCACN_IP_TCP:
-	case NCALRPC:
-		if (!s->binding->endpoint) {
-			struct composite_context *binding_req;
-			binding_req = dcerpc_epm_map_binding_send(c, s->binding, s->table,
-								  s->pipe->conn->event_ctx,
-								  s->lp_ctx);
-			composite_continue(c, binding_req, continue_map_binding, c);
-			return c;
-		}
-
-	default:
-		break;
 	}
 
 	continue_connect(c, s);
@@ -828,7 +1124,7 @@ _PUBLIC_ NTSTATUS dcerpc_pipe_connect_b_recv(struct composite_context *c, TALLOC
 */
 _PUBLIC_ NTSTATUS dcerpc_pipe_connect_b(TALLOC_CTX *parent_ctx,
 			       struct dcerpc_pipe **pp,
-			       struct dcerpc_binding *binding,
+			       const struct dcerpc_binding *binding,
 			       const struct ndr_interface_table *table,
 			       struct cli_credentials *credentials,
 			       struct tevent_context *ev,
