@@ -80,7 +80,7 @@ static NTSTATUS svfs_connect(struct ntvfs_module_context *ntvfs,
 	NT_STATUS_HAVE_NO_MEMORY(p);
 	p->ntvfs = ntvfs;
 	p->next_search_handle = 0;
-	p->connectpath = talloc_strdup(p, share_string_option(scfg, SHARE_PATH, ""));
+	p->connectpath = share_string_option(p, scfg, SHARE_PATH, "");
 	p->open_files = NULL;
 	p->search = NULL;
 
@@ -414,7 +414,10 @@ do_open:
 	NT_STATUS_NOT_OK_RETURN(status);
 
 	f = talloc(handle, struct svfs_file);
-	NT_STATUS_HAVE_NO_MEMORY(f);
+	if (f == NULL) {
+		close(fd);
+		return NT_STATUS_NO_MEMORY;
+	}
 	f->fd = fd;
 	f->name = talloc_strdup(f, unix_path);
 	NT_STATUS_HAVE_NO_MEMORY(f->name);
