@@ -58,47 +58,51 @@ static NTSTATUS sldb_init(TALLOC_CTX *mem_ctx, const struct share_ops *ops,
 	return NT_STATUS_OK;
 }
 
-static const char *sldb_string_option(struct share_config *scfg, const char *opt_name, const char *defval)
+static char *sldb_string_option(TALLOC_CTX *mem_ctx, struct share_config *scfg, const char *opt_name, const char *defval)
 {
 	struct ldb_message *msg;
 	struct ldb_message_element *el;
+	const char *colon;
 
-	if (scfg == NULL) return defval;
+	if (scfg == NULL) return talloc_strdup(mem_ctx, defval);
 
 	msg = talloc_get_type(scfg->opaque, struct ldb_message);
 
-	if (strchr(opt_name, ':')) {
-		char *name, *p;
+	colon = strchr(opt_name, ':');
+	if (colon != NULL) {
+		char *name;
 
 		name = talloc_strdup(scfg, opt_name);
 		if (!name) {
 			return NULL;
 		}
-		p = strchr(name, ':');
-		*p = '-';
+		name[colon-opt_name] = '-';
 
 		el = ldb_msg_find_element(msg, name);
+		TALLOC_FREE(name);
 	} else {
 		el = ldb_msg_find_element(msg, opt_name);
 	}
 
 	if (el == NULL) {
-		return defval;
+		return talloc_strdup(mem_ctx, defval);
 	}
 
-	return (const char *)(el->values[0].data);
+	return (char *)(el->values[0].data);
 }
 
 static int sldb_int_option(struct share_config *scfg, const char *opt_name, int defval)
 {
-	const char *val;
+	char *val;
 	int ret;
 
-       	val = sldb_string_option(scfg, opt_name, NULL);
+	val = sldb_string_option(scfg, scfg, opt_name, NULL);
 	if (val == NULL) return defval;
 
 	errno = 0;
 	ret = (int)strtol(val, NULL, 10);
+	TALLOC_FREE(val);
+
 	if (errno) return -1;
 
 	return ret;
@@ -106,13 +110,17 @@ static int sldb_int_option(struct share_config *scfg, const char *opt_name, int
 
 static bool sldb_bool_option(struct share_config *scfg, const char *opt_name, bool defval)
 {
-	const char *val;
+	char *val;
 
-       	val = sldb_string_option(scfg, opt_name, NULL);
+	val = sldb_string_option(scfg, scfg, opt_name, NULL);
 	if (val == NULL) return defval;
 
-	if (strcasecmp(val, "true") == 0) return true;
+	if (strcasecmp(val, "true") == 0) {
+		TALLOC_FREE(val);
+		return true;
+	}
 
+	TALLOC_FREE(val);
 	return false;
 }
 
@@ -121,23 +129,25 @@ static const char **sldb_string_list_option(TALLOC_CTX *mem_ctx, struct share_co
 	struct ldb_message *msg;
 	struct ldb_message_element *el;
 	const char **list;
+	const char *colon;
 	int i;
 
 	if (scfg == NULL) return NULL;
 
 	msg = talloc_get_type(scfg->opaque, struct ldb_message);
 
-	if (strchr(opt_name, ':')) {
-		char *name, *p;
+	colon = strchr(opt_name, ':');
+	if (colon != NULL) {
+		char *name;
 
 		name = talloc_strdup(scfg, opt_name);
 		if (!name) {
 			return NULL;
 		}
-		p = strchr(name, ':');
-		*p = '-';
+		name[colon-opt_name] = '-';
 
 		el = ldb_msg_find_element(msg, name);
+		TALLOC_FREE(name);
 	} else {
 		el = ldb_msg_find_element(msg, opt_name);
 	}
