@@ -936,7 +936,8 @@ static WERROR getncchanges_repl_secret(struct drsuapi_bind_state *b_state,
 				       bool has_get_all_changes)
 {
 	struct drsuapi_DsReplicaObjectIdentifier *ncRoot = req10->naming_context;
-	struct ldb_dn *obj_dn, *rodc_dn, *krbtgt_link_dn;
+	struct ldb_dn *obj_dn = NULL;
+	struct ldb_dn *rodc_dn, *krbtgt_link_dn;
 	int ret;
 	const char *rodc_attrs[] = { "msDS-KrbTgtLink", "msDS-NeverRevealGroup", "msDS-RevealOnDemandGroup", NULL };
 	const char *obj_attrs[] = { "tokenGroups", "objectSid", "UserAccountControl", "msDS-KrbTgtLinkBL", NULL };
