@@ -200,7 +200,7 @@ NTSTATUS tstream_setup_named_pipe(TALLOC_CTX *mem_ctx,
 		goto fail;
 	}
 
-	if (!directory_create_or_exist(lpcfg_ncalrpc_dir(lp_ctx), geteuid(), 0755)) {
+	if (!directory_create_or_exist(lpcfg_ncalrpc_dir(lp_ctx), 0755)) {
 		status = map_nt_error_from_unix_common(errno);
 		DEBUG(0,(__location__ ": Failed to create ncalrpc pipe directory '%s' - %s\n",
 			 lpcfg_ncalrpc_dir(lp_ctx), nt_errstr(status)));
