@@ -441,6 +441,7 @@ static bool queryinfokey_in_check(struct torture_context *tctx, struct winreg_Qu
 	return true;
 }
 
+#if 0
 static const uint8_t queryinfokey_out_data[] = {
   0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x05, 0x00, 0x00, 0x00,
   0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -459,6 +460,7 @@ static bool queryinfokey_out_check(struct torture_context *tctx, struct winreg_Q
 	torture_assert_werr_ok(tctx, r->out.result, "return code");
 	return true;
 }
+#endif
 
 static const uint8_t notifychangekeyvalue_in_data[] = {
   0x00, 0x00, 0x00, 0x00, 0xb2, 0x64, 0xbc, 0xb3, 0x7f, 0x90, 0x29, 0x4a,
@@ -489,6 +491,7 @@ static bool notifychangekeyvalue_out_check(struct torture_context *tctx, struct
 	return true;
 }
 
+#if 0
 static const uint8_t getkeysecurity_in_data[] = {
   0x00, 0x00, 0x00, 0x00, 0xbd, 0xaa, 0xf6, 0x59, 0xc1, 0x82, 0x1f, 0x4d,
   0x84, 0xa9, 0xdd, 0xae, 0x60, 0x77, 0x1e, 0x45, 0x00, 0x00, 0x00, 0x02,
@@ -521,6 +524,7 @@ static bool getkeysecurity_out_check(struct torture_context *tctx,
 	torture_assert_werr_ok(tctx, r->out.result, "return code");
 	return true;
 }
+#endif
 
 static const uint8_t enumkey_in_data[] = {
   0x00, 0x00, 0x00, 0x00, 0x85, 0xb8, 0x41, 0xb0, 0x17, 0xe4, 0x28, 0x45,
