@@ -21,6 +21,7 @@
 #include "libcli/libcli.h"
 #include "libcli/raw/raw_proto.h"
 #include "torture/torture.h"
+#include "torture/unix/proto.h"
 #include "lib/cmdline/popt_common.h"
 #include "auth/credentials/credentials.h"
 #include "param/param.h"
@@ -345,7 +346,7 @@ bool torture_unix_whoami(struct torture_context *torture)
 {
 	struct smbcli_state *cli;
 	struct smb_whoami whoami;
-	bool ret;
+	bool ret = false;
 	struct ldb_context *ldb;
 	const char *addc, *host;
 
